from .edit_file import edit_file
from .search_web import search_web
from .completion import task_complete
from .search_arxiv import search_arxiv
from .repomap import get_code_repo_map
from .write_csv import append_row_to_csv
from .graph import (
    get_node_details,
    add_knowledge_node,
    delete_knowledge_node,
    rename_knowledge_node,
    move_knowledge_node,
    get_knowledge_graph_tree,
    add_tags_to_knowledge_node,
    remove_tags_from_knowledge_node,
)
from .request_input import request_admin_input
from .screenshot import save_screenshot_to_file
from .worker import worker, worker_gen, chatgroup
from .click import find_and_click_element, scroll_screen
from .subtasks import create_task, resume_task, get_all_tasks_status, get_task_result, create_tasks_from_csv
from .deep_search import deepsearch

#显式导入 aient.plugins 中的所需内容
from ..aient.aient.plugins import (
    get_time,
    read_file,
    read_image,
    register_tool,
    write_to_file,
    excute_command,
    generate_image,
    list_directory,
    get_url_content,
    run_python_script,
    set_readonly_path,
    get_search_results,
    download_read_arxiv_pdf,
)

__all__ = [
    "worker",
    "get_time",
    "edit_file",
    "read_file",
    "chatgroup",
    "worker_gen",
    "read_image",
    "search_web",
    "deepsearch",
    "create_task",
    "resume_task",
    "search_arxiv",
    "write_to_file",
    "scroll_screen",
    "register_tool",
    "task_complete",
    "excute_command",
    "generate_image",
    "list_directory",
    "get_task_result",
    "get_url_content",
    "get_node_details",
    "add_knowledge_node",
    "move_knowledge_node",
    "delete_knowledge_node",
    "rename_knowledge_node",
    "get_knowledge_graph_tree",
    "add_tags_to_knowledge_node",
    "remove_tags_from_knowledge_node",
    "append_row_to_csv",
    "set_readonly_path",
    "get_code_repo_map",
    "run_python_script",
    "get_search_results",
    "request_admin_input",
    "get_all_tasks_status",
    "create_tasks_from_csv",
    "find_and_click_element",
    "download_read_arxiv_pdf",
    "save_screenshot_to_file",
]


import pickle
import base64
import asyncio
import logging
import hashlib
import mimetypes
import uuid
import threading
from dataclasses import dataclass
from abc import ABC, abstractmethod
from typing import List, Dict, Any, Optional, Union, Callable

# A wrapper to manage multiple providers with the same name
class ProviderGroup:
    """A container for multiple providers that share the same name, allowing for bulk operations."""
    def __init__(self, providers: List['ContextProvider']):
        self._providers = providers
    def __getitem__(self, key: int) -> 'ContextProvider':
        """Allows accessing providers by index, e.g., group[-1]."""
        return self._providers[key]
    def __iter__(self):
        """Allows iterating over the providers."""
        return iter(self._providers)
    def __len__(self) -> int:
        """Returns the number of providers in the group."""
        return len(self._providers)
    @property
    def visible(self) -> List[bool]:
        """Gets the visibility of all providers in the group."""
        return [p.visible for p in self._providers]
    @visible.setter
    def visible(self, value: bool):
        """Sets the visibility for all providers in the group."""
        for p in self._providers:
            p.visible = value

# Global, thread-safe registry for providers created within f-strings
_fstring_provider_registry = {}
_registry_lock = threading.Lock()

def _register_provider(provider: 'ContextProvider') -> str:
    """Registers a provider and returns a unique placeholder."""
    with _registry_lock:
        provider_id = f"__provider_placeholder_{uuid.uuid4().hex}__"
        _fstring_provider_registry[provider_id] = provider
        return provider_id

def _retrieve_provider(placeholder: str) -> Optional['ContextProvider']:
    """Retrieves a provider from the registry."""
    with _registry_lock:
        return _fstring_provider_registry.pop(placeholder, None)

# 1. 核心数据结构: ContentBlock
@dataclass
class ContentBlock:
    name: str
    content: str

# 2. 上下文提供者 (带缓存)
class ContextProvider(ABC):
    def __init__(self, name: str, visible: bool = True):
        self.name = name
        self._cached_content: Optional[str] = None
        self._is_stale: bool = True
        self._visible: bool = visible

    def __str__(self):
        # This allows the object to be captured when used inside an f-string.
        return _register_provider(self)

    def mark_stale(self): self._is_stale = True

    @property
    def visible(self) -> bool:
        """Gets the visibility of the provider."""
        return self._visible

    @visible.setter
    def visible(self, value: bool):
        """Sets the visibility of the provider."""
        if self._visible != value:
            self._visible = value
            # Content needs to be re-evaluated, but the source data hasn't changed,
            # so just marking it stale is enough for the renderer to reconsider it.
            self.mark_stale()
    async def refresh(self):
        if self._is_stale:
            self._cached_content = await self.render()
            self._is_stale = False
    @abstractmethod
    async def render(self) -> Optional[str]: raise NotImplementedError
    @abstractmethod
    def update(self, *args, **kwargs): raise NotImplementedError
    def get_content_block(self) -> Optional[ContentBlock]:
        if self.visible and self._cached_content is not None:
            return ContentBlock(self.name, self._cached_content)
        return None

    def __add__(self, other):
        if isinstance(other, Message):
            # Create a new message of the same type as `other`, with `self` prepended.
            new_items = [self] + other.provider()
            return type(other)(*new_items)
        return NotImplemented

class Texts(ContextProvider):
    def __init__(self, text: Optional[Union[str, Callable[[], str]]] = None, name: Optional[str] = None, visible: bool = True, newline: bool = False):
        if text is None and name is None:
            raise ValueError("Either 'text' or 'name' must be provided.")
        self.newline = newline

        # Ensure that non-callable inputs are treated as strings
        if not callable(text):
            self._text = str(text) if text is not None else None
        else:
            self._text = text

        self._is_dynamic = callable(self._text)

        if name is None:
            if self._is_dynamic:
                import uuid
                _name = f"dynamic_text_{uuid.uuid4().hex[:8]}"
            else:
                # Handle the case where text is None during initialization
                h = hashlib.sha1(self._text.encode() if self._text else b'').hexdigest()
                _name = f"text_{h[:8]}"
        else:
            _name = name
        super().__init__(_name, visible=visible)
        if not self._is_dynamic:
            self._cached_content = self.content
            # The content is cached, but it's still "stale" from the perspective
            # of the async refresh cycle. Let the first refresh formalize it.
            self._is_stale = True

    async def refresh(self):
        if self._is_dynamic:
            self._is_stale = True
        await super().refresh()

    def update(self, text: Union[str, Callable[[], str]]):
        self._text = text
        self._is_dynamic = callable(self._text)
        self.mark_stale()

    @property
    def content(self) -> Optional[str]:
        """
        Synchronously retrieves the raw text content as a property.
        If the content is dynamic (a callable), it executes the callable.
        """
        if self._is_dynamic:
            # Ensure dynamic content returns a string, even if empty
            result = self._text()
            return result if result is not None else ""
        # Ensure static content returns a string, even if empty
        return self._text if self._text is not None else ""

    async def render(self) -> Optional[str]:
        return self.content

    def __getstate__(self):
        """Custom state for pickling."""
        state = self.__dict__.copy()
        if self._is_dynamic:
            # For dynamic content, we snapshot its current value for serialization.
            # The lambda function itself cannot be pickled.
            try:
                # Evaluate the lambda and store it as a static string
                state['_text'] = self.content
                # Mark it as no longer dynamic in the pickled state
                state['_is_dynamic'] = False
            except Exception as e:
                # If the lambda fails for some reason, store an error message.
                logging.error(f"Error evaluating dynamic text '{self.name}' during pickling: {e}")
                state['_text'] = f"[Error: Could not evaluate dynamic content during save: {e}]"
                state['_is_dynamic'] = False
        return state

    def __setstate__(self, state):
        """Custom state for unpickling."""
        # Just restore the dictionary. The transformation is one-way.
        self.__dict__.update(state)

    def __eq__(self, other):
        if not isinstance(other, Texts):
            return NotImplemented
        # If either object is dynamic, they are only equal if they are the exact same object.
        if self._is_dynamic or (hasattr(other, '_is_dynamic') and other._is_dynamic):
            return self is other
        # For static content, compare the actual content.
        return self.content == other.content

class Tools(ContextProvider):
    def __init__(self, tools_json: Optional[List[Dict]] = None, name: str = "tools", visible: bool = True):
        super().__init__(name, visible=visible)
        self._tools_json = tools_json or []
        # Pre-render and cache the content, but leave it stale for the first refresh
        if self._tools_json:
            self._cached_content = f"<tools>{str(self._tools_json)}</tools>"
        self._is_stale = True
    def update(self, tools_json: List[Dict]):
        self._tools_json = tools_json
        self.mark_stale()
    async def render(self) -> Optional[str]:
        if not self._tools_json:
            return None
        return f"<tools>{str(self._tools_json)}</tools>"

    def __eq__(self, other):
        if not isinstance(other, Tools):
            return NotImplemented
        return self._tools_json == other._tools_json

class Files(ContextProvider):
    def __init__(self, *paths: Union[str, List[str]], name: str = "files", visible: bool = True):
        super().__init__(name, visible=visible)
        self._files: Dict[str, str] = {}

        file_paths: List[str] = []
        if paths:
            # Handle the case where the first argument is a list of paths, e.g., Files(['a', 'b'])
            if len(paths) == 1 and isinstance(paths[0], list):
                file_paths.extend(paths[0])
            # Handle the case where arguments are individual string paths, e.g., Files('a', 'b')
            else:
                file_paths.extend(paths)

        if file_paths:
            for path in file_paths:
                try:
                    with open(path, 'r', encoding='utf-8') as f:
                        self._files[path] = f.read()
                except FileNotFoundError:
                    logging.warning(f"File not found during initialization: {path}. Skipping.")
                except Exception as e:
                    logging.error(f"Error reading file {path} during initialization: {e}")

    async def refresh(self):
        """
        Overrides the default refresh behavior. It synchronizes the content of
        all tracked files with the file system. If a file is not found, its
        content is updated to reflect the error.
        """
        is_changed = False
        for path in list(self._files.keys()):
            try:
                with open(path, 'r', encoding='utf-8') as f:
                    new_content = f.read()
                if self._files.get(path) != new_content:
                    self._files[path] = new_content
                    is_changed = True
            except FileNotFoundError:
                error_msg = f"[Error: File not found at path '{path}']"
                if self._files.get(path) != error_msg:
                    self._files[path] = error_msg
                    is_changed = True
            except Exception as e:
                error_msg = f"[Error: Could not read file at path '{path}': {e}]"
                if self._files.get(path) != error_msg:
                    self._files[path] = error_msg
                    is_changed = True

        if is_changed:
            self.mark_stale()

        await super().refresh()

    def update(self, path: str, content: Optional[str] = None):
        """
        Updates a single file. If content is provided, it updates the file in
        memory. If content is None, it reads the file from disk.
        """
        if content is not None:
            self._files[path] = content
        else:
            try:
                with open(path, 'r', encoding='utf-8') as f:
                    self._files[path] = f.read()
            except FileNotFoundError:
                logging.error(f"File not found for update: {path}.")
                self._files[path] = f"[Error: File not found at path '{path}']"
            except Exception as e:
                logging.error(f"Error reading file for update {path}: {e}.")
                self._files[path] = f"[Error: Could not read file at path '{path}': {e}]"
        self.mark_stale()
    async def render(self) -> str:
        if not self._files: return None
        return "<latest_file_content>" + "\n".join([f"<file><file_path>{p}</file_path><file_content>{c}</file_content></file>" for p, c in self._files.items()]) + "\n</latest_file_content>"

    def __eq__(self, other):
        if not isinstance(other, Files):
            return NotImplemented
        return self._files == other._files

class Images(ContextProvider):
    def __init__(self, url: str, name: Optional[str] = None, visible: bool = True):
        super().__init__(name or url, visible=visible)
        self.url = url
        if self.url.startswith("data:"):
            self._cached_content = self.url
        self._is_stale = True
    def update(self, url: str):
        self.url = url
        self.mark_stale()
    async def render(self) -> Optional[str]:
        if self.url.startswith("data:"):
            return self.url
        try:
            with open(self.url, "rb") as image_file:
                encoded_string = base64.b64encode(image_file.read()).decode('utf-8')
                mime_type, _ = mimetypes.guess_type(self.url)
                if not mime_type: mime_type = "application/octet-stream" # Fallback
                return f"data:{mime_type};base64,{encoded_string}"
        except FileNotFoundError:
            logging.warning(f"Image file not found: {self.url}. Skipping.")
            return None # Or handle error appropriately

    def __eq__(self, other):
        if not isinstance(other, Images):
            return NotImplemented
        return self.url == other.url

# 3. 消息类 (已合并 MessageContent)
class Message(ABC):
    def __init__(self, role: str, *initial_items: Union[ContextProvider, str, list, 'Message']):
        self.role = role
        processed_items = []
        for item in initial_items:
            if item is None:
                continue

            # This is the new recursive flattening logic
            if isinstance(item, Message):
                processed_items.extend(item.provider())
            elif isinstance(item, str):
                import re
                placeholder_pattern = re.compile(r'(__provider_placeholder_[a-f0-9]{32}__)')
                parts = placeholder_pattern.split(item)
                if len(parts) > 1:
                    for part in parts:
                        if not part: continue
                        if placeholder_pattern.match(part):
                            provider = _retrieve_provider(part)
                            if provider:
                                processed_items.append(provider)
                        else:
                            processed_items.append(Texts(text=part))
                else:
                    processed_items.append(Texts(text=item))
            elif isinstance(item, ContextProvider):
                processed_items.append(item)
            elif isinstance(item, list):
                for sub_item in item:
                    if not isinstance(sub_item, dict) or 'type' not in sub_item:
                        raise ValueError("List items must be dicts with a 'type' key.")
                    item_type = sub_item['type']
                    if item_type == 'text':
                        processed_items.append(Texts(text=sub_item.get('text', '')))
                    elif item_type == 'image_url':
                        image_url = sub_item.get('image_url', {}).get('url')
                        if image_url:
                            processed_items.append(Images(url=image_url))
                    else:
                        raise ValueError(f"Unsupported item type in list: {item_type}")
            else:
                raise TypeError(f"Unsupported item type: {type(item)}. Must be str, ContextProvider, or list.")
        self._items: List[ContextProvider] = processed_items
        self._parent_messages: Optional['Messages'] = None

    @property
    def content(self) -> Optional[Union[str, List[Dict[str, Any]]]]:
        """
        Renders the message content.
        For simple text messages, returns a string.
        For multimodal messages, returns a list of content blocks.
        """
        rendered_dict = self.to_dict()
        return rendered_dict.get('content') if rendered_dict else None

    def _render_content(self) -> str:
        final_parts = []
        for item in self._items:
            block = item.get_content_block()
            if block and block.content is not None:
                # Check if it's a Texts provider with newline=True
                # and it's not the very first item with content.
                if isinstance(item, Texts) and hasattr(item, 'newline') and item.newline and final_parts:
                    final_parts.append("\n\n")
                final_parts.append(block.content)
        return "".join(final_parts)

    def pop(self, name: str) -> Optional[ContextProvider]:
        popped_item = None
        for i, item in enumerate(self._items):
            if hasattr(item, 'name') and item.name == name:
                popped_item = self._items.pop(i)
                break
        if popped_item and self._parent_messages:
            self._parent_messages._notify_provider_removed(popped_item)
        return popped_item

    def insert(self, index: int, item: ContextProvider):
        self._items.insert(index, item)
        if self._parent_messages:
            self._parent_messages._notify_provider_added(item, self)

    def append(self, item: ContextProvider):
        self._items.append(item)
        if self._parent_messages:
            self._parent_messages._notify_provider_added(item, self)

    def provider(self, name: Optional[str] = None) -> Optional[Union[ContextProvider, ProviderGroup, List[ContextProvider]]]:
        if name is None:
            return self._items

        named_providers = [p for p in self._items if hasattr(p, 'name') and p.name == name]

        if not named_providers:
            return None
        if len(named_providers) == 1:
            return named_providers[0]
        return ProviderGroup(named_providers)

    def __add__(self, other):
        if isinstance(other, str):
            new_items = self._items + [Texts(text=other)]
            return type(self)(*new_items)
        if isinstance(other, Message):
            new_items = self._items + other.provider()
            return type(self)(*new_items)
        return NotImplemented

    def __radd__(self, other):
        if isinstance(other, str):
            new_items = [Texts(text=other)] + self._items
            return type(self)(*new_items)
        if isinstance(other, Message):
            new_items = other.provider() + self._items
            return type(self)(*new_items)
        return NotImplemented

    def __getitem__(self, key: Union[str, int]) -> Any:
        """
        使得 Message 对象支持字典风格的访问 (e.g., message['content'])
        和列表风格的索引访问 (e.g., message[-1])。
        """
        if isinstance(key, str):
            if key == 'role':
                return self.role
            elif key == 'content':
                # 直接调用 to_dict 并提取 'content'，确保逻辑一致
                rendered_dict = self.to_dict()
                return rendered_dict.get('content') if rendered_dict else None
            # 对于 tool_calls 等特殊属性，也通过 to_dict 获取
            elif hasattr(self, key):
                rendered_dict = self.to_dict()
                if rendered_dict and key in rendered_dict:
                    return rendered_dict[key]

            # 如果在对象本身或其 to_dict() 中都找不到，则引发 KeyError
            if hasattr(self, key):
                 return getattr(self, key)
            raise KeyError(f"'{key}'")
        elif isinstance(key, int):
            return self._items[key]
        else:
            raise TypeError(f"Message indices must be integers or strings, not {type(key).__name__}")

    def __len__(self) -> int:
        """返回消息中 provider 的数量。"""
        return len(self._items)

    def __repr__(self): return f"Message(role='{self.role}', items={[i.name for i in self._items]})"

    def __contains__(self, item: Any) -> bool:
        """Checks if a ContextProvider is in the message."""
        if not isinstance(item, ContextProvider):
            return False
        # The `in` operator on a list checks for equality,
        # and our custom __eq__ on ContextProvider handles the comparison logic.
        return item in self._items

    def has(self, provider_type: type) -> bool:
        """Checks if the message contains a provider of a specific type."""
        if not isinstance(provider_type, type) or not issubclass(provider_type, ContextProvider):
            raise TypeError("provider_type must be a subclass of ContextProvider")
        return any(isinstance(p, provider_type) for p in self._items)

    def lstrip(self, provider_type: type):
        """
        从消息的左侧（开头）移除所有指定类型的 provider。
        移除操作会一直持续，直到遇到一个不同类型的 provider 为止。
        """
        while self._items and type(self._items[0]) is provider_type:
            self.pop(self._items[0].name)

    def rstrip(self, provider_type: type):
        """
        从消息的右侧（末尾）移除所有指定类型的 provider。
        移除操作会一直持续，直到遇到一个不同类型的 provider 为止。
        """
        while self._items and type(self._items[-1]) is provider_type:
            self.pop(self._items[-1].name)

    def strip(self, provider_type: type):
        """
        从消息的两侧移除所有指定类型的 provider。
        """
        self.lstrip(provider_type)
        self.rstrip(provider_type)

    def __bool__(self) -> bool:
        return bool(self._items)
    def get(self, key: str, default: Any = None) -> Any:
        """提供类似字典的 .get() 方法来访问属性。"""
        return getattr(self, key, default)

    async def refresh(self):
        """刷新此消息中的所有 provider。"""
        tasks = [provider.refresh() for provider in self._items]
        await asyncio.gather(*tasks)

    async def render(self) -> Optional[Dict[str, Any]]:
        """
        渲染消息为字典。首次调用时会隐式刷新以确保动态内容被加载。
        后续调用将返回缓存版本，除非手动调用了 refresh()。
        """
        # 检查是否是首次渲染
        is_first_render = not all(hasattr(p, '_cached_content') and p._cached_content is not None for p in self._items if p._is_stale)

        if is_first_render:
            await self.refresh()

        return self.to_dict()

    async def render_latest(self) -> Optional[Dict[str, Any]]:
        """始终刷新并返回最新的渲染结果。"""
        await self.refresh()
        return self.to_dict()

    def to_dict(self) -> Optional[Dict[str, Any]]:
        is_multimodal = any(isinstance(p, Images) for p in self._items)

        if not is_multimodal:
            rendered_content = self._render_content()
            if not rendered_content: return None
            return {"role": self.role, "content": rendered_content}
        else:
            content_list = []
            for item in self._items:
                block = item.get_content_block()
                if not block or not block.content: continue
                if isinstance(item, Images):
                    content_list.append({"type": "image_url", "image_url": {"url": block.content}})
                else:
                    content_list.append({"type": "text", "text": block.content})
            if not content_list: return None
            return {"role": self.role, "content": content_list}

class SystemMessage(Message):
    def __init__(self, *items): super().__init__("system", *items)
class UserMessage(Message):
    def __init__(self, *items): super().__init__("user", *items)
class AssistantMessage(Message):
    def __init__(self, *items): super().__init__("assistant", *items)

class RoleMessage:
    """A factory class that creates a specific message type based on the role."""
    def __new__(cls, role: str, *items):
        if role == 'system':
            return SystemMessage(*items)
        elif role == 'user':
            return UserMessage(*items)
        elif role == 'assistant':
            return AssistantMessage(*items)
        else:
            raise ValueError(f"Invalid role: {role}. Must be 'system', 'user', or 'assistant'.")

class ToolCalls(Message):
    """Represents an assistant message that requests tool calls."""
    def __init__(self, tool_calls: List[Any]):
        super().__init__("assistant")
        self.tool_calls = tool_calls

    def to_dict(self) -> Dict[str, Any]:
        # Duck-typing serialization for OpenAI's tool_call objects
        serialized_calls = []
        for tc in self.tool_calls:
            try:
                # Attempt to serialize based on openai-python > 1.0 tool_call structure
                func = tc.function
                serialized_calls.append({
                    "id": tc.id,
                    "type": tc.type,
                    "function": { "name": func.name, "arguments": func.arguments }
                })
            except AttributeError:
                if isinstance(tc, dict):
                    serialized_calls.append(tc) # Assume it's already a serializable dict
                else:
                    raise TypeError(f"Unsupported tool_call type: {type(tc)}. It should be an OpenAI tool_call object or a dict.")

        return {
            "role": self.role,
            "tool_calls": serialized_calls,
            "content": None
        }

class ToolResults(Message):
    """Represents a tool message with the result of a single tool call."""
    def __init__(self, tool_call_id: str, content: Union[str, Message]):
        # The base Message class now handles the absorption of a Message object.
        # We just need to pass the content to the parent __init__.
        # For ToolResults, we primarily care about the textual content.
        if isinstance(content, Message):
             # Extract only text-like providers to pass to the parent
            text_providers = [p for p in content.provider() if not isinstance(p, Images)]
            super().__init__("tool", *text_providers)
        else:
            super().__init__("tool", content)

        self.tool_call_id = tool_call_id
        # After initialization, render the content to a simple string for _content.
        self._content = self._render_content()

    def to_dict(self) -> Dict[str, Any]:
        return {
            "role": self.role,
            "tool_call_id": self.tool_call_id,
            "content": self._content
        }

# 4. 顶层容器: Messages
class Messages:
    def __init__(self, *initial_messages: Message):
        from typing import Tuple
        self._messages: List[Message] = []
        self._providers_index: Dict[str, List[Tuple[ContextProvider, Message]]] = {}
        if initial_messages:
            for msg in initial_messages:
                self.append(msg)

    def _notify_provider_added(self, provider: ContextProvider, message: Message):
        if provider.name not in self._providers_index:
            self._providers_index[provider.name] = []
        self._providers_index[provider.name].append((provider, message))

    def _notify_provider_removed(self, provider: ContextProvider):
        if provider.name in self._providers_index:
            # Create a new list excluding the provider to be removed.
            # Comparing by object identity (`is`) is crucial here.
            providers_list = self._providers_index[provider.name]
            new_list = [(p, m) for p, m in providers_list if p is not provider]

            if not new_list:
                # If the list becomes empty, remove the key from the dictionary.
                del self._providers_index[provider.name]
            else:
                # Otherwise, update the dictionary with the new list.
                self._providers_index[provider.name] = new_list

    def provider(self, name: str) -> Optional[Union[ContextProvider, ProviderGroup]]:
        indexed_list = self._providers_index.get(name)
        if not indexed_list:
            return None

        providers = [p for p, m in indexed_list]
        if len(providers) == 1:
            return providers[0]
        else:
            return ProviderGroup(providers)

    def pop(self, key: Optional[Union[str, int]] = None) -> Union[Optional[ContextProvider], Optional[Message]]:
        # If no key is provided, pop the last message.
        if key is None:
            key = len(self._messages) - 1

        if isinstance(key, str):
            indexed_list = self._providers_index.get(key)
            if not indexed_list:
                return None
            # Pop the first one found, which is consistent with how pop usually works
            _provider, parent_message = indexed_list[0]
            # The actual removal from _providers_index happens in _notify_provider_removed
            # which is called by message.pop()
            return parent_message.pop(key)
        elif isinstance(key, int):
            try:
                if key < 0: # Handle negative indices like -1
                    key += len(self._messages)
                if not (0 <= key < len(self._messages)):
                    return None
                popped_message = self._messages.pop(key)
                popped_message._parent_messages = None
                for provider in popped_message.provider():
                    self._notify_provider_removed(provider)
                return popped_message
            except IndexError:
                return None

        return None

    async def refresh(self):
        tasks = []
        for provider_list in self._providers_index.values():
            for provider, _ in provider_list:
                tasks.append(provider.refresh())
        await asyncio.gather(*tasks)

    def render(self) -> List[Dict[str, Any]]:
        results = [msg.to_dict() for msg in self._messages]
        return [res for res in results if res]

    async def render_latest(self) -> List[Dict[str, Any]]:
        await self.refresh()
        return self.render()

    def append(self, message: Message):
        if self._messages and self._messages[-1].role == message.role:
            last_message = self._messages[-1]
            for provider in message.provider():
                last_message.append(provider)
        else:
            message._parent_messages = self
            self._messages.append(message)
            for p in message.provider():
                self._notify_provider_added(p, message)

    def save(self, file_path: str):
        """
        Saves the entire Messages object to a file using pickle.
        Warning: Deserializing data with pickle from an untrusted source is insecure.
        """
        with open(file_path, 'wb') as f:
            pickle.dump(self, f)

    @classmethod
    def load(cls, file_path: str) -> Optional['Messages']:
        """
        Loads a Messages object from a file using pickle.
        Returns the loaded object, or None if the file is not found or an error occurs.
        Warning: Only load files from a trusted source.
        """
        try:
            with open(file_path, 'rb') as f:
                return pickle.load(f)
        except FileNotFoundError:
            # logging.warning(f"File not found at {file_path}, returning empty Messages.")
            return cls()
        except (pickle.UnpicklingError, EOFError) as e:
            logging.error(f"Could not deserialize file {file_path}: {e}")
            return cls()

    def __getitem__(self, index: Union[int, slice]) -> Union[Message, 'Messages']:
        if isinstance(index, slice):
            return Messages(*self._messages[index])
        return self._messages[index]

    def __setitem__(self, index: Union[int, slice], value: Union[Message, 'Messages']):
        if isinstance(index, int):
            if not isinstance(value, Message):
                raise TypeError("When assigning to an index, the value must be a Message.")

            if not (-len(self._messages) <= index < len(self._messages)):
                raise IndexError("Messages assignment index out of range")

            # Get old message to remove its providers
            old_message = self._messages[index]
            for provider in old_message.provider():
                self._notify_provider_removed(provider)

            # Assign new message
            self._messages[index] = value
            value._parent_messages = self
            for provider in value.provider():
                self._notify_provider_added(provider, value)

        elif isinstance(index, slice):
            if not isinstance(value, Messages):
                raise TypeError("When assigning to a slice, the value must be a Messages object.")

            start, stop, step = index.indices(len(self._messages))
            if step != 1:
                raise ValueError("Slice assignment with step is not supported.")

            # Remove old providers from the index
            for i in range(start, stop):
                for provider in self._messages[i].provider():
                    self._notify_provider_removed(provider)

            # Replace the slice in the list
            self._messages[start:stop] = value._messages

            # Add new providers to the index and set parent
            for msg in value:
                msg._parent_messages = self
                for provider in msg.provider():
                    self._notify_provider_added(provider, msg)
        else:
            raise TypeError("Unsupported operand type(s) for assignment")

    def __len__(self) -> int: return len(self._messages)
    def __iter__(self): return iter(self._messages)
    def __repr__(self):
        return f"Messages({repr(self._messages)})"
    def __contains__(self, item: Any) -> bool:
        """Checks if a Message or ContextProvider is in the collection."""
        if isinstance(item, Message):
            # Check for object identity
            return any(item is msg for msg in self._messages)
        if isinstance(item, ContextProvider):
            # Check if any message contains the provider
            return any(item in msg for msg in self._messages)
        return False



import contextvars
from .broker import MessageBroker
from .bemcp.bemcp import MCPManager
from .taskmanager import TaskManager
from .knowledge_graph import KnowledgeGraphManager

"""
全局共享实例
"""

broker = MessageBroker()
mcp_manager = MCPManager()
kgm = KnowledgeGraphManager(broker=broker)
current_task_manager = contextvars.ContextVar('current_task_manager')
current_work_dir = contextvars.ContextVar('current_work_dir', default=None)

def get_task_manager():
    """Creates a new, isolated TaskManager instance."""
    return TaskManager()



import asyncio
import functools
from typing import Any, Callable, TypeVar

try:
    from mcp.shared.exceptions import McpError
except ImportError:
    # Define a dummy exception if mcp is not available,
    # although it should be in the target environment.
    class McpError(Exception):
        pass

try:
    from anyio import BrokenResourceError
except ImportError:
    class BrokenResourceError(Exception):
        pass

F = TypeVar('F', bound=Callable[..., Any])

def async_retry(max_retries: int = 2, delay: float = 1.0):
    """
    A decorator to automatically retry an async function if it raises an exception.

    Args:
        max_retries: The maximum number of retries.
        delay: The delay between retries in seconds.
    """
    def decorator(func: F) -> F:
        @functools.wraps(func)
        async def wrapper(*args: Any, **kwargs: Any) -> Any:
            last_exception = None
            # The number of attempts is max_retries + 1 (the initial attempt)
            for attempt in range(max_retries + 1):
                try:
                    return await func(*args, **kwargs)
                except Exception as e:
                    last_exception = e
                    if attempt < max_retries:
                        print(f"Attempt {attempt + 1} failed with error: {e}. Retrying in {delay}s...")
                        await asyncio.sleep(delay)
                    else:
                        print(f"All {max_retries + 1} attempts failed.")
            if last_exception is not None:
                raise last_exception
        return wrapper  # type: ignore
    return decorator


def reconnect_on_connection_error(func: F) -> F:
    """
    A decorator for MCPClient methods that automatically tries to reconnect
    and retry the call if a connection-related error is caught.
    It handles McpError and anyio.BrokenResourceError.
    """
    @functools.wraps(func)
    async def wrapper(self, *args: Any, **kwargs: Any) -> Any:
        try:
            return await func(self, *args, **kwargs)
        except (McpError, BrokenResourceError) as e:
            is_connection_error = False
            if isinstance(e, McpError):
                error_str = str(e).lower()
                if "connection closed" in error_str or "peer closed connection" in error_str:
                    is_connection_error = True
            elif isinstance(e, BrokenResourceError):
                is_connection_error = True

            if is_connection_error:
                print(f"Connection error detected ({type(e).__name__}): {e}. Attempting to reconnect...")
                try:
                    await self.disconnect()
                    await self.connect()
                    print("Reconnected successfully. Retrying the operation...")
                    return await func(self, *args, **kwargs)
                except Exception as reconnect_e:
                    print(f"Failed to reconnect and retry: {reconnect_e}")
                    # If reconnect fails, raise the original connection error
                    raise e
            else:
                # Not a connection-related McpError, re-raise it.
                raise
    return wrapper # type: ignore


from ..aient.aient.plugins import register_tool

@register_tool()
def task_complete(message: str) -> str:
    """
    当任务完成时，调用此工具以返回最终结果。

    这个工具接收一个表示任务完成信息的字符串，并将其直接返回。
    它标志着一个任务的成功结束，并将最终的输出传递给用户或调用者。

    Args:
        message (str): 任务完成的信息或最终结果。必填字段。

    Returns:
        str: 传入的任务完成信息。
    """
    return message



import os
import pkgutil
import importlib

# 首先导入registry，因为其他模块中的装饰器依赖它
from .registry import registry, register_tool, register_agent

# 自动导入当前目录下所有的插件模块
excluded_modules = ['config', 'registry', '__init__']
current_dir = os.path.dirname(__file__)

# 先导入所有模块，确保装饰器被执行
for _, module_name, _ in pkgutil.iter_modules([current_dir]):
    if module_name not in excluded_modules:
        importlib.import_module(f'.{module_name}', package=__name__)

# 然后从config导入必要的定义
from .config import *

# 确保将所有工具函数添加到全局名称空间
for tool_name, tool_func in registry.tools.items():
    globals()[tool_name] = tool_func

__all__ = [
    'PLUGINS',
    'function_call_list',
    'get_tools_result_async',
    'registry',
    'register_tool',
    'register_agent',
    'update_tools_config',
    'get_function_call_list',
] + list(registry.tools.keys())


from ..utils.scripts import sandbox

from .registry import register_tool

@register_tool()
def set_readonly_path(path: str):
    """
    将指定的文件或目录路径设置为只读。

    这个函数将一个路径添加到只读路径列表中。一旦被设置为只读，
    任何对该路径的写操作都将被禁止。

    Args:
        path (str): 需要设置为只读的文件或目录的路径。

    Returns:
        str: 操作成功或失败的提示信息。
    """
    status = sandbox.add_readonly_path(path)
    if status == "Success":
        return f"路径 '{path}' 已成功设置为只读。"
    else:
        return f"系统未启用该功能，无法设置只读路径。"


from datetime import datetime

from ..aient.aient.plugins import (
    # read_file,
    # read_image,
    register_tool,
    write_to_file,
    excute_command,
    # list_directory,
    # get_url_content,
    # run_python_script,
    # set_readonly_path,
    # get_search_results,
    # download_read_arxiv_pdf,
)


from beswarm.tools import (
    # worker,
    # get_code_repo_map,
    # search_arxiv,
    # edit_file,
    # request_admin_input,
    search_web,
    create_task,
    get_task_result,
    get_all_tasks_status,
    resume_task,
    create_tasks_from_csv,
    get_node_details,
    add_knowledge_node,
    delete_knowledge_node,
    rename_knowledge_node,
    move_knowledge_node,
    get_knowledge_graph_tree,
    add_tags_to_knowledge_node,
    remove_tags_from_knowledge_node,
)

from ..core import mcp_manager, broker, kgm, get_task_manager, current_task_manager, current_work_dir
from ..agents.planact import BrokerWorker


@register_tool()
def get_task1_goal(query: str):
    """
    根据用户查询生成任务1的目标。

    这个函数为第一个子任务创建一个详细的提示。该子任务的目标是
    使用 `search_web` 工具搜索与查询相关的信息，然后使用
    `create_tasks_from_csv` 工具根据搜索结果创建更多的子任务。

    返回的字符串是一个模版，将用作初始任务的“目标”。
    它包含了指导AI代理（扮演“首席情报官”）从网络搜索结果构建知识图谱的说明。

    参数:
        query: 用户的搜索查询。

    返回:
        一个格式化的字符串，包含任务的详细目标提示。
    """
    return (
        f"用户查询：{query}"
        """
调用 search_web 工具搜索，该工具会把搜索结果保存到 csv 文件中。

**禁止**读取任何 csv 文件。

然后使用 create_tasks_from_csv 工具，从 csv 文件中创建多个子任务。

csv 的 title 是 query, url和content，content是url对应的网页内容。query 是查询内容。

下面是通过 create_tasks_from_csv 创建子任务需要传入的 goal 模版，请将```内字符串原样传入，不要修改：

```
### **Deep Search & Synthesis Prompt (深度搜索与综合 Prompt)**

**核心任务:** 你是一位**首席情报官、信息整合大师与战略分析师**。你的唯一目标是，围绕核心查询 (`{query}`), 通过深度分析指定URL (`{url}`) 的内容 (在 `<content>` 标签内), 动态地构建并填充一个**充满洞察、结构清晰、并具备决策价值**的知识图谱。

**【决策者画像】**
你的用户是一位**聪明的领域分析师、研究员或战略决策者**。他们需要一个关于 `{query}` 的、超越了简单链接列表的、结构化的深度理解。你的输出必须兼顾信息的精确性和整体格局的洞察力。

**【第一原则 / The Prime Directive: 将信息转化为可行动的战略洞察】**
1.  **【价值创造】** 你的目标远不止于信息的摘要，而是**提炼全新的理解、识别关键联系、并揭示潜在的洞察**。这要求你必须做到：**识别核心论点与反方观点**、**提取关键数据与证据**、**映射关键实体（人物、组织）及其关系**、**指明新兴趋势与待解问题**。
2.  **【结构即大纲与查询根节点】** 整个知识图谱**必须且只能有一个根节点**，它代表核心查询 `{query}`。所有信息都必须是此根节点的子孙节点。当分析新内容时，应将其**核心观点**填充到**根节点下已有的主题节点**中（例如“核心论点”或“关键数据”），而非创建重复的结构。
3.  **【来源可溯源】** 图谱中的**每一个信息点**都必须明确标注其来源 `#source:{url}`。
4.  **【整合优先，审慎创建】** 尽一切努力将新信息归入现有的大纲结构中。只有当一个观点在逻辑上完全无法融入当前结构时，才考虑调整大纲（例如，新增一个“争议焦点”的三级标题）。
5.  **【拥抱涌现】** 对 `{query}` 的理解是在持续分析中动态生长和重构的有机体，要为未来的可能性保留空间。
6.  **【类别与实例分离】** **二级、三级节点必须是抽象的标题** (如 “关键人物”, “支持性证据”)。**具体的人物、组织、数据点等实例，只能作为叶子节点存在于各级标题之下。**
7.  **【层级为骨，标签为魂】** 使用**层级结构**来表示信息的**主题归属关系**。对于所有**次要的、跨领域的、多维度的属性** (如观点倾向、证据类型)，**必须**使用**标签**来表示。

**【命名风格规范】**
所有节点的名称**必须**采用 **“中文名称 (英文全称或常用缩写)”** 的格式。

**输入信息:**
- **查询 (Query):** {query}
- **URL:** {url}
- **网页内容 (Content):** 在 `<content>` 标签内
<content>
{content}
</content>

**核心工作流:**

你必须严格遵循以下**五个步骤**的思维与行动模型来完成任务：

1.  **【第1步：审视全局与校准意图 (Review Landscape & Calibrate Intent)】**
    *   调用 `get_knowledge_graph_tree` 工具，深入理解当前围绕 `{query}` 已构建的**知识图谱结构、核心主题、以及已有的标签体系**。明确当前分析的主要目的是为了填补哪些信息空白。

2.  **【第2步：快速甄别与多维深潜 (Triage & Multi-dimensional Deep Dive)】**
    *   **快速甄别:** 首先，判断该 `{url}` 的 `content` 是否与 `{query}` **直接相关**。如果完全无关，则**立即调用 `task_complete` 并注明“内容不相关”**，终止任务。
    *   **多维深潜:** 如果相关，则必须从以下**多个维度**对内容进行强制性的深度分析：
        *   **核心论点评估 (Central Thesis Assessment):** 此内容针对 `{query}` 提出了什么**核心观点、主张或关键信息**？
        *   **证据与数据提取 (Evidence & Data Extraction):** 内容中是否包含可量化的**数据、统计、关键性能指标或强力证据**？必须将其作为独立节点提取。(标签: `#数据点`, `#统计数据`, `#案例研究`)
        *   **立场与偏见识别 (Stance & Bias Identification):** 作者或来源对 `{query}` 持什么立场？是**支持、反对，还是中立**？是否存在明显的**商业或意识形态偏见**？ (标签: `#支持观点`, `#反对观点`, `#中立分析`, `#商业推广`)
        *   **关键实体识别 (Key Entity Recognition):** 内容中是否提到了与 `{query}` 相关的**关键人物、组织、产品或技术**？这些应被识别并可能成为独立的节点。
        *   **观点链接 (Argument Interconnectivity):** 此内容的观点是否**印证、反驳或补充**了知识图谱中已有的某个观点？
        *   **信源强度评估 (Source Strength Assessment):** 该来源的**权威性**如何？是**官方报告、专家分析、新闻报道，还是用户生成内容**？ (标签: `#权威信源`, `#专家意见`, `#媒体报道`, `#用户评论`)
        *   **洞察提炼 (Actionable Insight Distillation):** 该信息是否揭示了某个**新兴趋势、潜在风险、商业机会或有待进一步研究的问题**？ (标签: `#新兴趋势`, `#潜在风险`, `#未来方向`)

3.  **【第3步：精确整合 (Precise Integration)】**
    *   根据你的多维分析，调用工具 (`add_knowledge_node`, `add_tags_to_knowledge_node` 等) 将内容的知识点精确地整合进图谱。
    *   **必须立即**为所有新节点打上**来源标签 (`#source:{url}`)** 和在第2步中规划好的所有**分析标签**。

4.  **【第4步：自我审查与价值综合 (Self-Correction & Value Synthesis)】**
    *   **在你调用`task_complete`之前，必须执行这一最终的、强制性的审查与综合步骤。**
    *   再次调用 `get_knowledge_graph_tree()`，审视整个知识图谱，并对照以下的**【高价值情报简报清单】**进行重构：
        *   **检查项1 (来源可溯源性):** 图谱中的**每一个信息点**是否都已正确、无遗漏地标注了其 `#source:{url}`？
        *   **检查项2 (结构一致性):** 整个知识图谱是否严格维持在**单一查询根节点**之下？我是否将新信息正确**填充**到了已有的主题中，而不是创建了重复的类别？
        *   **检查项3 (观点综合呈现):** 我是否已将**对立或互补的观点**组织在专门的标题下（如“争议焦点”）进行**对比呈现**？
        *   **检查项4 (数据集中化):** 我能否创建一个对比性的列表或小节，来**并列呈现不同来源的关键数据**？
        *   **检查项5 (关键实体网络):** 我是否清晰地展示了**核心实体（人物、组织）及其之间的关系**？
        *   **检查项6 (洞察凸显):** **最具价值的洞察** (例如，被标记为 `#新兴趋势` 或 `#潜在风险` 的节点) 是否被组织在了专门的“**核心摘要**”或“**战略展望**”章节下，使其一目了然？
    *   **如果检查发现任何可以提升情报价值的机会，你必须立即调用相应工具进行重构。**

5.  **【第5步：完成任务 (Finalize)】**
    *   只有当你确认知识图谱（情报简报）在**结构、逻辑、深度和洞察力**上都达到了你作为**首席情报官**的最高标准时，才能调用 `task_complete` 结束你的任务。
```

观察知识图谱，如果当前信息不足以回答用户的查询，则继续从不同角度搜索，直到信息足够为止。继续循环上一个子任务，直到信息足够为止。
        """
    )

@register_tool()
def get_task2_goal(query: str):
    """
    根据用户查询生成任务2的目标。

    这个函数为第二个子任务创建一个详细的提示。该子任务的目标是
    扮演一个“首席分析师”的角色，基于第一个任务构建的知识图谱，
    生成一份最终的情报分析报告。

    返回的字符串是一个模版，将用作第二个主要任务的“目标”。
    它包含了指导AI代理如何审视知识图谱、构建报告大纲、
    通过创建更多子任务来并行撰写报告的各个章节、
    最终将所有章节整合成一份完整报告的详细说明。

    参数:
        query: 用户的搜索查询。

    返回:
        一个格式化的字符串，包含任务的详细目标提示。
    """
    return (
        f"用户查询：{query}"
        """
### **Deep Synthesis & Report Generation Prompt (深度综合与报告生成 Prompt)**

**任务:** 你是一位顶尖的**首席分析师**和资深的**情报报告总编辑**。你和你的情报官团队已经围绕核心查询 (`{query}`)，成功构建了一个高度结构化、信息丰富的知识图谱。现在，你的最终任务是基于这个完美的知识图谱，撰写一篇全面、深入、且具备决策价值的**最终情报分析报告**。

**【最高指导原则：结构即洞察，深度源于综合】**
1.  **以图谱为唯一蓝本:** 你撰写的报告，其**章节、段落、乃至论述的逻辑顺序，都必须严格遵循我们已经构建好的知识图谱树状结构**。图谱的层级就是你报告的层级。
2.  **融合细节，提炼洞察:** 不要简单地罗列图谱中的叶子节点信息（如单个数据或观点）。你的任务是**综合**一个类别下的所有实例（叶子节点），并**提炼**出该类别的**核心趋势、关键发现、主要争论点或战略启示**。
3.  **引用来源，彰显可信:** 在论述中提及任何具体信息、数据或观点时，你**必须**利用记录在节点`tags`中的`#source`信息，以清晰的方式（如 `(Source: {url})`）来标注来源。

**你的工作流程:**

你将采用一种**“总编规划 -> 分析师并行撰写 -> 总编整合终审”**的专业报告生成工作流。

1.  **【第1步：全局情报审视 (Global Intelligence Review)】**
    *   调用 `get_knowledge_graph_tree`，获取最终的、完整的知识图谱。这是你撰写报告的唯一事实来源，也是你分发给手下分析师们的“共享数据库”。

2.  **【第2步：构建报告大纲并分派任务 (Report Outline & Task Delegation)】**
    *   **a. 构思报告框架**: 基于你对整个知识图谱的理解，创造一个符合专业情报报告范式、具有逻辑递进关系的**报告大纲（即报告目录）**。例如，可以包含“执行摘要”、“核心发现”、“关键实体分析”、“数据与证据”、“新兴趋势与风险”、“战略展望”等章节。
    *   **b. 精准定位素材**: 对于大纲中的**每一个章节**，明确指出它对应知识图谱中的**哪一个或哪几个主要分支**。这是你接下来要分派给每个分析师的“撰写范围”。
    *   **c. 分派撰写子任务**: 基于你的大纲，调用`create_task`工具，为**每一个章节**创建一个并行的子任务。你**必须**将以下信息清晰地传递给每个子任务的`goal`中：
        *   **`section_title`**: 你为该章节设定的标题。
        *   **`list_of_relevant_kg_paths`**: 你在(b)步中为该章节划定的、在知识图谱中对应的源路径列表。
        *   **`full_report_outline`**: 完整的报告大纲，让每个分析师都了解自己的工作在整个报告中的位置，以便更好地把握上下文和撰写口吻。
        *   **`chief_analyst_notes`**: （可选但推荐）你作为总编辑对本章撰写的特别指示，例如“本章重点要突出不同来源观点的冲突与矛盾”或“请将本章数据进行可视化呈现的建议”。

3.  **【第3步：监控与等待 (Monitor & Wait)】**
    *   所有章节的撰写任务已并行派出。你的任务是监控它们的进度，并使用`get_task_result`工具，以等待所有撰写任务全部完成。

4.  **【第4步：整合与终审 (Integration & Final Review)】**
    *   调用`get_task_result`返回了所有子任务的结果，现在你的手上有多份独立的Markdown章节草稿。
    *   你的任务是：
        a. **整合**: 将所有章节草稿按照你最初设计的大纲顺序，合并成一篇完整的分析报告。
        b. **审校**: 通读全文，检查章节间的过渡是否流畅，术语使用是否一致，整体风格是否统一。进行必要的编辑和润色。
        c. **撰写执行摘要与战略展望**: 基于所有章节的深度内容，亲自撰写全文的**“执行摘要”**（提炼核心洞察给决策者）和**“战略展望”**（指明未来方向和建议）部分，为整篇报告注入灵魂。

5.  **【第5步：完成任务 (Finalize)】**
    *   当你对这篇由团队协作完成、并由你亲自统稿的报告感到满意时，编写执行摘要和战略展望章节，编写脚本通过 excute_command 使用 cat 和输出重定向进行文件内容拼接将所有章节合并到同一个md文件保存到本地，最后调用 `task_complete`，并将**最终的、完整的Markdown报告全文的文件路径**作为`message`参数返回。

---
### **【分析师专用】子任务`goal`** (注意：这是子任务的`goal`，填入必要信息后，传给子-任务执行)

**任务:** 你是一位专业的领域分析师。你的总编辑已经为你规划好了整篇报告的结构，并为你分配了具体的章节撰写任务。

**你的撰写任务:**

- **章节标题:** `{section_title}` (例如: "3. 关键实体分析")
- **撰写范围 (知识图谱路径):** `{list_of_relevant_kg_paths}` (例如: `["{query}/关键人物", "{query}/相关组织"]`)

**核心指令:**

1.  **获取全局图谱**: 首先，调用 `get_knowledge_graph_tree` 获取完整的、最新的知识图谱，作为你的参考背景。

2.  **聚焦与深挖**: 将你的注意力**严格限制**在你被分配的撰写范围（`{list_of_relevant_kg_paths}`）内。
    *   遍历这些分支下的所有叶子节点。
    *   对于每一个重要的叶子节点，调用 `get_node_details` 来获取其完整的`description`（其中包含了核心信息）和`#source`标签。

3.  **提炼与叙述**: 基于你收集到的详细信息，撰写一段**只关于本章节内容**的、逻辑连贯、语言流畅的Markdown文本保存到本地。
    *   你需要综合信息，提炼观点（例如，总结某一类实体的共同特征或相互关系），而不是简单地罗列事实。
    *   在论述中提及任何具体信息或数据时，**必须**以 `(Source: {url})` 的格式清晰地标注其来源。
    *   如果需要，可以使用Markdown的表格来对比不同来源的数据。

4.  **串行写作:** 你将以串行的方式，逐一完成每一个二级标题的撰写，分阶段写入本地文件。对于每个需要撰写的二级标题的内容，查看知识图谱分析是否存在可以补充的背景信息，调用 get_node_details 获取信息来补充。

5.  **对于每一个二级标题，你必须遵循以下写作规范:**
    *   a. **规划段落:** 在动笔前，先构思好要写2-3个段落来阐述观点。**禁止**使用不必要的无序列表。
    *   b. **详细阐述:** 确保内容详实、有据可依、逻辑清晰，确保篇幅适中，每个二级标题只有一段是不合适的。
    *   c. **承上启下:** 在开始撰写一个新的二级标题时，你的第一句话应该巧妙地与上一个标题的内容进行关联或对比，以确保章节的流畅性。

6.  **完成本章**: 当你完成了本章节的撰写后，调用 `task_complete`，并将**你撰写的本章节Markdown文本的文件路径**作为`message`参数返回。**不要包含任何额外的内容，只返回你负责的这一章。**
        """
    )

tools = [
    write_to_file,
    excute_command,
    search_web,
    create_task,
    get_task_result,
    get_all_tasks_status,
    resume_task,
    create_tasks_from_csv,
    get_knowledge_graph_tree,
    add_knowledge_node,
    add_tags_to_knowledge_node,
    rename_knowledge_node,
    move_knowledge_node,
    get_node_details,
    get_task1_goal,
    get_task2_goal,
]

@register_tool()
async def deepsearch(query: str, work_dir: str):
    """
    执行一个深度搜索和综合任务，分为两个主要阶段。

    deepsearch 任务通过编排两个连续的子任务来响应用户查询。
    首先，它启动一个“情报收集”阶段，然后是一个“报告生成”阶段。

    参数:
        query (str): 用户的原始搜索查询。
        work_dir (str): 工作目录的绝对路径。deepsearch 任务将在此目录上下文中执行操作。deepsearch 任务的工作目录位置在主任务的工作目录的子目录。deepsearch 任务工作目录**禁止**设置为主任务目录本身。

    返回:
        worker执行的结果。
    """
    start_time = datetime.now()
    task_manager = get_task_manager()
    current_task_manager.set(task_manager)
    current_work_dir.set(work_dir)

    goal = f"""
**禁止**执行 get_task1_goal，get_task2_goal。

使用 create_task 启动第一个子任务，下面是 goal 原文，请将```内字符串原样传入，不要修改：

```
用户查询：{query}

使用get_task1_goal获取任务安排。
```

使用 create_task 第一个子任务结束后，启动第二个子任务，第二个子任务需要 excute_command 工具，下面是 goal 原文，请将```内字符串原样传入，不要修改：

```
用户查询：{query}

使用get_task2_goal获取任务安排。

务必先执行第一个子任务，等第一个子任务完成后，再执行第二个子任务。
等待两个子任务都完成后，才调用 task_complete 结束任务。
```
"""

    worker_instance = BrokerWorker(goal, tools, work_dir, True, broker, mcp_manager, task_manager, kgm)
    result = await worker_instance.run()
    end_time = datetime.now()
    print(f"\n任务开始时间: {start_time.strftime('%Y-%m-%d %H:%M:%S')}")
    print(f"任务结束时间: {end_time.strftime('%Y-%m-%d %H:%M:%S')}")
    print(f"总用时: {end_time - start_time}")
    return result


import asyncio

from mcp.client.stdio import stdio_client
from mcp import ClientSession, StdioServerParameters

# 为 stdio 连接创建服务器参数
server_params = StdioServerParameters(
    # 服务器执行的命令，这里我们使用 uv 来运行 web_search.py
    command='uv',
    # 运行的参数
    args=['run', 'server.py'],
    # 环境变量，默认为 None，表示使用当前环境变量
    # env=None
)


async def main():
    # 创建 stdio 客户端
    async with stdio_client(server_params) as (stdio, write):
        # 创建 ClientSession 对象
        async with ClientSession(stdio, write) as session:
            # 初始化 ClientSession
            await session.initialize()

            # 列出可用的工具
            response = await session.list_tools()
            print(response.model_dump_json(indent=2))

            response = await session.list_resources()
            print(response.model_dump_json(indent=2))

            response = await session.list_prompts()
            print(response.model_dump_json(indent=2))

            # 调用工具
            response = await session.call_tool('add', {"a": "2", "b": "3"})
            print(response.model_dump_json(indent=2))

            # 调用工具
            response = await session.read_resource('config://app')
            print(response.model_dump_json(indent=2))

            # 调用工具
            response = await session.get_prompt('debug_error', {"error": "test"})
            print(response.model_dump_json(indent=2))


if __name__ == '__main__':
    asyncio.run(main())



import os
import uuid
import asyncio
from enum import Enum
from pathlib import Path

# 1. 定义你的协程任务函数
async def worker_task_async(goal):
    """一个模拟工作的异步任务函数"""
    print(f"任务: 开始执行，预计耗时 {str(goal)} 秒...")
    # 必须使用 asyncio.sleep() 而不是 time.sleep()
    await asyncio.sleep(goal)
    result = "任务完成！"
    print("-> 任务: 已完成。")
    return result

class TaskStatus(Enum):
    """任务状态枚举"""
    PENDING = "PENDING"
    RUNNING = "RUNNING"
    DONE = "DONE"
    CANCELLED = "CANCELLED"
    ERROR = "ERROR"
    NOT_FOUND = "NOT_FOUND"





class TaskManager:
    """一个简单的异步任务管理器"""
    def __init__(self):
        self.tasks = {}  # 使用字典来存储任务，key是task_id, value是task对象
        self.results_queue = asyncio.Queue()
        self.root_path = Path(os.getcwd())

    def create_tasks(self, task_coro, tasks_params):
        """
        批量创建并注册任务。

        Args:
            task_coro: 用于创建任务的协程函数。
            tasks_params (list): 包含任务参数的列表。

        Returns:
            list: 创建的任务ID列表。
        """
        task_ids = []
        for args in tasks_params:
            coro = task_coro(**args)
            task_id = self.create_task(coro)
            task_ids.append(task_id)
        return task_ids

    def create_task(self, coro):
        """
        创建并注册一个新任务。

        Args:
            coro: 要执行的协程。
            name (str, optional): 任务的可读名称。 Defaults to None.

        Returns:
            str: 任务的唯一ID。
        """
        task_id = str(uuid.uuid4())
        task_name = f"Task-{task_id[:8]}"

        # 使用 asyncio.create_task() 创建任务
        task = asyncio.create_task(coro, name=task_name)

        # 将任务存储在管理器中
        # 当任务完成时，通过回调函数将结果放入队列
        task.add_done_callback(
            lambda t: self._on_task_done(task_id, t)
        )
        self.tasks[task_id] = task
        print(f"任务已创建: ID={task_id}, Name={task_name}")
        return task_id

    def get_task_status(self, task_id):
        """
        查询特定任务的状态。

        Args:
            task_id (str): 要查询的任务ID。

        Returns:
            TaskStatus: 任务的当前状态。
        """
        task = self.tasks.get(task_id)
        if not task:
            return TaskStatus.NOT_FOUND

        if task.done():
            if task.cancelled():
                return TaskStatus.CANCELLED
            elif task.exception() is not None:
                return TaskStatus.ERROR
            else:
                return TaskStatus.DONE

        # asyncio.Task 没有直接的 'RUNNING' 状态。
        # 如果任务还没有完成，它要么是等待执行（PENDING），要么是正在执行（RUNNING）。
        # 这里我们简化处理，认为未完成的就是运行中。
        return TaskStatus.RUNNING

    def get_task_result(self, task_id):
        """获取已完成任务的结果，如果任务未完成或出错则返回相应信息。"""
        task = self.tasks.get(task_id)
        if self.get_task_status(task_id) == TaskStatus.DONE:
            return task.result()
        elif self.get_task_status(task_id) == TaskStatus.ERROR:
            return task.exception()
        return None

    def _on_task_done(self, task_id, task):
        """私有回调函数，在任务完成时将结果放入队列。"""
        try:
            # 将元组 (task_id, status, result) 放入队列
            self.results_queue.put_nowait(
                (task_id, TaskStatus.DONE, task.result())
            )
        except asyncio.CancelledError:
            self.results_queue.put_nowait(
                (task_id, TaskStatus.CANCELLED, None)
            )
        except Exception as e:
            self.results_queue.put_nowait(
                (task_id, TaskStatus.ERROR, e)
            )

    async def get_next_result(self):
        """
        等待并返回下一个完成的任务结果。

        如果所有任务都已提交，但没有任务完成，此方法将异步等待。

        Returns:
            tuple: 一个包含 (task_id, status, result) 的元组。
        """
        return await self.results_queue.get()

    def get_task_index(self, task_id):
        """
        获取任务在任务字典中的插入顺序索引。

        Args:
            task_id (str): 要查询的任务ID。

        Returns:
            int: 任务的索引（从0开始），如果未找到则返回-1。
        """
        try:
            # 将字典的键转换为列表并查找索引
            task_ids_list = list(self.tasks.keys())
            return task_ids_list.index(task_id)
        except ValueError:
            # 如果任务ID不存在，则返回-1
            return -1

    def set_root_path(self, root_path):
        self.root_path = Path(root_path)

async def main():
    manager = TaskManager()

    # --- 任务提交阶段 ---
    print("--- 任务提交 ---")

    tasks_to_run = [
        {"goal": 3},
        {"goal": 1},
        {"goal": 5},
        {"goal": 2},
        {"goal": 4},
    ]
    task_ids = manager.create_tasks(worker_task_async, tasks_to_run)
    print(f"\n主程序: {len(task_ids)} 个任务已提交，现在开始等待结果...\n")

    # --- 结果处理阶段 ---
    # 使用 get_next_result() 逐个获取已完成任务的结果
    print("--- 结果处理 ---")
    for i in range(len(task_ids)):
        print(f"等待第 {i + 1} 个任务完成...")
        # 此处会异步等待，直到队列中有可用的结果
        task_id, status, result = await manager.get_next_result()

        # 从管理器中获取任务名称（如果需要）
        task_name = manager.tasks[task_id].get_name()

        print(f"-> 收到结果: 任务 {task_name}, index: {manager.get_task_index(task_id)}")
        print(f"  - 状态: {status.value}")

        if status == TaskStatus.DONE:
            print(f"  - 结果: '{result}'")
        elif status == TaskStatus.ERROR:
            print(f"  - 错误: {result}")
        elif status == TaskStatus.CANCELLED:
            print("  - 结果: 任务被取消")
        print("-" * 20)

    print("\n--- 所有任务的结果都已处理完毕 ---")

    tasks_to_run = [
        {"goal": 30},
        {"goal": 10},
        {"goal": 50},
        {"goal": 20},
        {"goal": 40},
    ]
    task_ids = manager.create_tasks(worker_task_async, tasks_to_run)
    print(f"\n主程序: {len(task_ids)} 个任务已提交，现在开始等待结果...\n")

    # --- 结果处理阶段 ---
    # 使用 get_next_result() 逐个获取已完成任务的结果
    print("--- 结果处理 ---")
    for i in range(len(task_ids)):
        print(f"等待第 {i + 1} 个任务完成...")
        # 此处会异步等待，直到队列中有可用的结果
        task_id, status, result = await manager.get_next_result()

        # 从管理器中获取任务名称（如果需要）
        task_name = manager.tasks[task_id].get_name()

        print(f"-> 收到结果: 任务 {task_name}, index: {manager.get_task_index(task_id)}")
        print(f"  - 状态: {status.value}")

        if status == TaskStatus.DONE:
            print(f"  - 结果: '{result}'")
        elif status == TaskStatus.ERROR:
            print(f"  - 错误: {result}")
        elif status == TaskStatus.CANCELLED:
            print("  - 结果: 任务被取消")
        print("-" * 20)

    print("\n--- 所有任务的结果都已处理完毕 ---")


# 运行主协程
if __name__ == "__main__":
    asyncio.run(main())
    print("\n主程序: 所有任务都已完成并处理。")



import os
import json
import chardet
from pdfminer.high_level import extract_text

from .registry import register_tool

# 读取文件内容
@register_tool()
def read_file(file_path, head: int = None):
    """
Description: Request to read the contents of a file at the specified path. Use this when you need to examine the contents of an existing file you do not know the contents of, for example to analyze code, review text files, or extract information from configuration files. Automatically extracts raw text from PDF and DOCX files. May not be suitable for other types of binary files, as it returns the raw content as a string.

注意：
1. pdf 文件 必须使用 read_file 读取，可以使用 read_file 直接读取 PDF。

参数:
    file_path: 要读取的文件路径，(required) The path of the file to read (relative to the current working directory)
    head: (可选) 读取文件的前N行，默认为None，读取整个文件

返回:
    文件内容的字符串

Usage:
<read_file>
<file_path>File path here</file_path>
</read_file>

Examples:

1. Reading an entire file:
<read_file>
<file_path>frontend.pdf</file_path>
</read_file>

2. Reading multiple files:

<read_file>
<file_path>frontend-config.json</file_path>
</read_file>

<read_file>
<file_path>backend-config.txt</file_path>
</read_file>

...

<read_file>
<file_path>README.md</file_path>
</read_file>
    """
    try:
        # 检查文件是否存在
        if not os.path.exists(file_path):
            return f"<tool_error>文件 '{file_path}' 不存在</tool_error>"

        # 检查是否为文件
        if not os.path.isfile(file_path):
            return f"<tool_error>'{file_path}' 不是一个文件</tool_error>"

        # 检查文件扩展名
        if file_path.lower().endswith('.pdf'):
            # 提取PDF文本
            text_content = extract_text(file_path)

            # 如果提取结果为空
            if not text_content:
                return f"<tool_error>无法从 '{file_path}' 提取文本内容</tool_error>"
        elif file_path.lower().endswith('.ipynb'):
            try:
                with open(file_path, 'r', encoding='utf-8') as file:
                    notebook_content = json.load(file)

                for cell in notebook_content.get('cells', []):
                    if cell.get('cell_type') == 'code' and 'outputs' in cell:
                        filtered_outputs = []
                        for output in cell.get('outputs', []):
                            new_output = output.copy()
                            if 'data' in new_output:
                                original_data = new_output['data']
                                filtered_data = {}
                                for key, value in original_data.items():
                                    if key.startswith('image/'):
                                        continue
                                    if key == 'text/html':
                                        html_content = "".join(value) if isinstance(value, list) else value
                                        if isinstance(html_content, str) and '<table class="show_videos"' in html_content:
                                            continue
                                    filtered_data[key] = value
                                if filtered_data:
                                    new_output['data'] = filtered_data
                                    filtered_outputs.append(new_output)
                            elif 'output_type' in new_output and new_output['output_type'] in ['stream', 'error']:
                                filtered_outputs.append(new_output)

                        cell['outputs'] = filtered_outputs

                text_content = json.dumps(notebook_content, indent=2, ensure_ascii=False)
            except json.JSONDecodeError:
                return f"<tool_error>文件 '{file_path}' 不是有效的JSON格式 (IPython Notebook)。</tool_error>"
            except Exception as e:
                return f"<tool_error>处理IPython Notebook文件 '{file_path}' 时发生错误: {e}</tool_error>"
        else:
            # 更新：修改通用文件读取逻辑以支持多种编码
            # 这部分替换了原有的 else 块内容
            try:
                with open(file_path, 'rb') as file: # 以二进制模式读取
                    raw_data = file.read()

                if not raw_data: # 处理空文件
                    text_content = ""
                else:
                    detected_info = chardet.detect(raw_data)
                    primary_encoding_to_try = detected_info['encoding']
                    confidence = detected_info['confidence']

                    decoded_successfully = False

                    # 尝试1: 使用检测到的编码 (如果置信度高且编码有效)
                    if primary_encoding_to_try and confidence > 0.7: # 您可以根据需要调整置信度阈值
                        try:
                            text_content = raw_data.decode(primary_encoding_to_try)
                            decoded_successfully = True
                        except (UnicodeDecodeError, LookupError): # LookupError 用于处理无效的编码名称
                            # 解码失败，将尝试后备编码
                            pass

                    # 尝试2: UTF-8 (如果第一次尝试失败或未进行)
                    if not decoded_successfully:
                        try:
                            text_content = raw_data.decode('utf-8')
                            decoded_successfully = True
                        except UnicodeDecodeError:
                            # 解码失败，将尝试下一个后备编码
                            pass

                    # 尝试3: UTF-16 (如果之前的尝试都失败)
                    # 'utf-16' 会处理带BOM的LE/BE编码。若无BOM，则假定为本机字节序。
                    # chardet 通常能更准确地检测具体的 utf-16le 或 utf-16be。
                    if not decoded_successfully:
                        try:
                            text_content = raw_data.decode('utf-16')
                            decoded_successfully = True
                        except UnicodeDecodeError:
                            # 所有主要尝试都失败
                            pass

                    if not decoded_successfully:
                        # 所有尝试均失败后的错误信息
                        detected_str_part = ""
                        if primary_encoding_to_try and confidence > 0.7: # 如果有高置信度的检测结果
                            detected_str_part = f"检测到的编码 '{primary_encoding_to_try}' (置信度 {confidence:.2f}), "
                        elif primary_encoding_to_try: # 如果有检测结果但置信度低
                            detected_str_part = f"低置信度检测编码 '{primary_encoding_to_try}' (置信度 {confidence:.2f}), "

                        return f"<tool_error>文件 '{file_path}' 无法解码。已尝试: {detected_str_part}UTF-8, UTF-16。</tool_error>"

            except FileNotFoundError:
                # 此处不太可能触发 FileNotFoundError，因为函数开头已有 os.path.exists 检查
                return f"<tool_error>文件 '{file_path}' 在读取过程中未找到。</tool_error>"
            except Exception as e:
                # 捕获在此块中可能发生的其他错误，例如未被早期检查捕获的文件读取问题
                return f"<tool_error>处理通用文件 '{file_path}' 时发生错误: {e}</tool_error>"

        if head is not None:
            try:
                num_lines = int(head)
                if num_lines > 0:
                    lines = text_content.splitlines(True)
                    return "".join(lines[:num_lines])
            except (ValueError, TypeError):
                # Invalid head value, ignore and proceed with normal logic.
                pass

        # if file_path.lower().endswith('.csv'):
        #     lines = text_content.splitlines(True)
        #     if len(lines) > 500:
        #         top_lines = lines[:250]
        #         bottom_lines = lines[-250:]
        #         omitted_count = len(lines) - 500
        #         text_content = "".join(top_lines) + f"\n... (中间省略了 {omitted_count} 行) ...\n" + "".join(bottom_lines)

        # 返回文件内容
        return text_content

    except PermissionError:
        return f"<tool_error>没有权限访问文件 '{file_path}'</tool_error>"
    except UnicodeDecodeError:
        # 更新：修改全局 UnicodeDecodeError 错误信息使其更通用
        return f"<tool_error>文件 '{file_path}' 包含无法解码的字符 (UnicodeDecodeError)。</tool_error>"
    except Exception as e:
        return f"<tool_error>读取文件时发生错误: {e}</tool_error>"

if __name__ == "__main__":
    # python -m beswarm.aient.aient.plugins.read_file
    result = read_file("./work/cax/Lenia Notebook.ipynb", head=10)
    print(result)
    print(len(result))



import re
import json
import copy
import httpx
import base64
import asyncio
from io import IOBase
from typing import Tuple
from urllib.parse import urlparse

from .models import RequestModel, Message
from .utils import (
    c3s,
    c3o,
    c3h,
    c35s,
    c4,
    gemini1,
    gemini2,
    gemini2_5_pro_exp,
    BaseAPI,
    safe_get,
    get_engine,
    get_model_dict,
    get_text_message,
    get_image_message,
)

gemini_max_token_65k_models = ["gemini-2.5-pro", "gemini-2.0-pro", "gemini-2.0-flash-thinking", "gemini-2.5-flash"]

async def get_gemini_payload(request, engine, provider, api_key=None):
    headers = {
        'Content-Type': 'application/json'
    }

    # 获取映射后的实际模型ID
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]

    if request.stream:
        gemini_stream = "streamGenerateContent"
    else:
        gemini_stream = "generateContent"
    url = provider['base_url']
    parsed_url = urlparse(url)
    if "/v1beta" in parsed_url.path:
        api_version = "v1beta"
    else:
        api_version = "v1"

    url = f"{parsed_url.scheme}://{parsed_url.netloc}{parsed_url.path.split('/models')[0].rstrip('/')}/models/{original_model}:{gemini_stream}?key={api_key}"

    messages = []
    systemInstruction = None
    system_prompt = ""
    function_arguments = None

    try:
        request_messages = [Message(role="user", content=request.prompt)]
    except:
        request_messages = copy.deepcopy(request.messages)
    for msg in request_messages:
        if msg.role == "assistant":
            msg.role = "model"
        tool_calls = None
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
                elif item.type == "image_url" and provider.get("image", True):
                    image_message = await get_image_message(item.image_url.url, engine)
                    content.append(image_message)
        elif msg.content:
            content = [{"text": msg.content}]
        elif msg.content is None:
            tool_calls = msg.tool_calls

        if tool_calls:
            tool_call = tool_calls[0]
            function_arguments = {
                "functionCall": {
                    "name": tool_call.function.name,
                    "args": json.loads(tool_call.function.arguments)
                }
            }
            messages.append(
                {
                    "role": "model",
                    "parts": [function_arguments]
                }
            )
        elif msg.role == "tool":
            function_call_name = function_arguments["functionCall"]["name"]
            messages.append(
                {
                    "role": "function",
                    "parts": [{
                    "functionResponse": {
                        "name": function_call_name,
                        "response": {
                            "name": function_call_name,
                            "content": {
                                "result": msg.content,
                            }
                        }
                    }
                    }]
                }
            )
        elif msg.role != "system" and content:
            messages.append({"role": msg.role, "parts": content})
        elif msg.role == "system":
            content[0]["text"] = re.sub(r"_+", "_", content[0]["text"])
            system_prompt = system_prompt + "\n\n" + content[0]["text"]
    if system_prompt.strip():
        systemInstruction = {"parts": [{"text": system_prompt}]}

    if any(off_model in original_model for off_model in gemini_max_token_65k_models) or original_model.endswith("-image-generation"):
        safety_settings = "OFF"
    else:
        safety_settings = "BLOCK_NONE"

    payload = {
        "contents": messages or [{"role": "user", "parts": [{"text": "No messages"}]}],
        "safetySettings": [
            {
                "category": "HARM_CATEGORY_HARASSMENT",
                "threshold": safety_settings
            },
            {
                "category": "HARM_CATEGORY_HATE_SPEECH",
                "threshold": safety_settings
            },
            {
                "category": "HARM_CATEGORY_SEXUALLY_EXPLICIT",
                "threshold": safety_settings
            },
            {
                "category": "HARM_CATEGORY_DANGEROUS_CONTENT",
                "threshold": safety_settings
            },
            {
                "category": "HARM_CATEGORY_CIVIC_INTEGRITY",
                "threshold": "BLOCK_NONE"
            },
        ]
    }

    if systemInstruction:
        if api_version == "v1beta":
            payload["systemInstruction"] = systemInstruction
        if api_version == "v1":
            first_message = safe_get(payload, "contents", 0, "parts", 0, "text", default=None)
            system_instruction = safe_get(systemInstruction, "parts", 0, "text", default=None)
            if first_message and system_instruction:
                payload["contents"][0]["parts"][0]["text"] = system_instruction + "\n" + first_message

    miss_fields = [
        'model',
        'messages',
        'stream',
        'tool_choice',
        'presence_penalty',
        'frequency_penalty',
        'n',
        'user',
        'include_usage',
        'logprobs',
        'top_logprobs',
        'response_format',
        'stream_options',
        'prompt',
        'size',
    ]
    generation_config = {}

    def process_tool_parameters(data):
        if isinstance(data, dict):
            # 移除 Gemini 不支持的 'additionalProperties'
            data.pop("additionalProperties", None)

            # 将 'default' 值移入 'description'
            if "default" in data:
                default_value = data.pop("default")
                description = data.get("description", "")
                data["description"] = f"{description}\nDefault: {default_value}"

            # 递归处理
            for value in data.values():
                process_tool_parameters(value)
        elif isinstance(data, list):
            for item in data:
                process_tool_parameters(item)

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            if field == "tools" and ("gemini-2.0-flash-thinking" in original_model or "gemini-2.5-flash-image" in original_model):
                continue
            if field == "tools":
                # 处理每个工具的 function 定义
                processed_tools = []
                for tool in value:
                    function_def = tool["function"]
                    if "parameters" in function_def:
                        process_tool_parameters(function_def["parameters"])

                    if function_def["name"] != "googleSearch" and function_def["name"] != "googleSearch":
                        processed_tools.append({"function": function_def})

                if processed_tools:
                    payload.update({
                        "tools": [{
                            "function_declarations": [tool["function"] for tool in processed_tools]
                        }],
                        "tool_config": {
                            "function_calling_config": {
                                "mode": "AUTO"
                            }
                        }
                    })
            elif field == "temperature":
                if "gemini-2.5-flash-image" in original_model:
                    value = 1
                generation_config["temperature"] = value
            elif field == "max_tokens":
                if value > 65536:
                    value = 65536
                generation_config["maxOutputTokens"] = value
            elif field == "top_p":
                generation_config["topP"] = value
            else:
                payload[field] = value

    payload["generationConfig"] = generation_config
    if "maxOutputTokens" not in generation_config:
        if any(pro_model in original_model for pro_model in gemini_max_token_65k_models):
            payload["generationConfig"]["maxOutputTokens"] = 65536
        else:
            payload["generationConfig"]["maxOutputTokens"] = 8192

        if original_model.endswith("-image-generation"):
            payload["generationConfig"]["response_modalities"] = [
                "Text",
                "Image",
            ]

    if "gemini-2.5" in original_model and "gemini-2.5-flash-image" not in original_model:
        # 从请求模型名中检测思考预算设置
        m = re.match(r".*-think-(-?\d+)", request.model)
        if m:
            try:
                val = int(m.group(1))
                budget = None
                # gemini-2.5-pro: [128, 32768]
                if "gemini-2.5-pro" in original_model:
                    if val < 128:
                        budget = 128
                    elif val > 32768:
                        budget = 32768
                    else: # 128 <= val <= 32768
                        budget = val

                # gemini-2.5-flash-lite: [0] or [512, 24576]
                elif "gemini-2.5-flash-lite" in original_model:
                    if val > 0 and val < 512:
                        budget = 512
                    elif val > 24576:
                        budget = 24576
                    else: # Includes 0 and valid range, and clamps invalid negatives
                        budget = val if val >= 0 else 0

                # gemini-2.5-flash (and other gemini-2.5 models as a fallback): [0, 24576]
                else:
                    if val > 24576:
                        budget = 24576
                    else: # Includes 0 and valid range, and clamps invalid negatives
                        budget = val if val >= 0 else 0

                payload["generationConfig"]["thinkingConfig"] = {
                    "includeThoughts": True if budget else False,
                    "thinkingBudget": budget
                }
            except ValueError:
                # 如果转换为整数失败，忽略思考预算设置
                pass
        else:
            payload["generationConfig"]["thinkingConfig"] = {
                "includeThoughts": True,
            }

    if safe_get(provider, "preferences", "post_body_parameter_overrides", default=None):
        for key, value in safe_get(provider, "preferences", "post_body_parameter_overrides", default={}).items():
            if key == request.model:
                for k, v in value.items():
                    payload[k] = v
            elif all(_model not in request.model.lower() for _model in ["gemini", "gpt", "claude", "deepseek"]) and "-" not in key:
                payload[key] = value

    return url, headers, payload

import time
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.serialization import load_pem_private_key

def create_jwt(client_email, private_key):
    # JWT Header
    header = json.dumps({
        "alg": "RS256",
        "typ": "JWT"
    }).encode()

    # JWT Payload
    now = int(time.time())
    payload = json.dumps({
        "iss": client_email,
        "scope": "https://www.googleapis.com/auth/cloud-platform",
        "aud": "https://oauth2.googleapis.com/token",
        "exp": now + 3600,
        "iat": now
    }).encode()

    # Encode header and payload
    segments = [
        base64.urlsafe_b64encode(header).rstrip(b'='),
        base64.urlsafe_b64encode(payload).rstrip(b'=')
    ]

    # Create signature
    signing_input = b'.'.join(segments)
    private_key = load_pem_private_key(private_key.encode(), password=None)
    signature = private_key.sign(
        signing_input,
        padding.PKCS1v15(),
        hashes.SHA256()
    )

    segments.append(base64.urlsafe_b64encode(signature).rstrip(b'='))
    return b'.'.join(segments).decode()

async def get_access_token(client_email, private_key):
    jwt = await asyncio.to_thread(create_jwt, client_email, private_key)

    async with httpx.AsyncClient() as client:
        response = await client.post(
            "https://oauth2.googleapis.com/token",
            data={
                "grant_type": "urn:ietf:params:oauth:grant-type:jwt-bearer",
                "assertion": jwt
            },
            headers={'Content-Type': "application/x-www-form-urlencoded"}
        )
        response.raise_for_status()
        return response.json()["access_token"]

async def get_vertex_gemini_payload(request, engine, provider, api_key=None):
    headers = {
        'Content-Type': 'application/json'
    }
    if provider.get("client_email") and provider.get("private_key"):
        access_token = await get_access_token(provider['client_email'], provider['private_key'])
        headers['Authorization'] = f"Bearer {access_token}"
    if provider.get("project_id"):
        project_id = provider.get("project_id")

    if request.stream:
        gemini_stream = "streamGenerateContent"
    else:
        gemini_stream = "generateContent"
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    # search_tool = None

    # https://cloud.google.com/vertex-ai/generative-ai/docs/models/gemini/2-0-flash?hl=zh-cn
    pro_models = ["gemini-2.5"]
    if any(pro_model in original_model for pro_model in pro_models):
        location = gemini2_5_pro_exp
    else:
        location = gemini1

    if "google-vertex-ai" in provider.get("base_url", ""):
        url = provider.get("base_url").rstrip('/') + "/v1/projects/{PROJECT_ID}/locations/{LOCATION}/publishers/google/models/{MODEL_ID}:{stream}".format(
            LOCATION=await location.next(),
            PROJECT_ID=project_id,
            MODEL_ID=original_model,
            stream=gemini_stream
        )
    elif api_key is not None and api_key[2] == ".":
        url = f"https://aiplatform.googleapis.com/v1/publishers/google/models/{original_model}:{gemini_stream}?key={api_key}"
        headers.pop("Authorization", None)
    else:
        url = "https://{LOCATION}-aiplatform.googleapis.com/v1/projects/{PROJECT_ID}/locations/{LOCATION}/publishers/google/models/{MODEL_ID}:{stream}".format(
            LOCATION=await location.next(),
            PROJECT_ID=project_id,
            MODEL_ID=original_model,
            stream=gemini_stream
        )

    messages = []
    systemInstruction = None
    system_prompt = ""
    function_arguments = None
    request_messages = copy.deepcopy(request.messages)
    for msg in request_messages:
        if msg.role == "assistant":
            msg.role = "model"
        tool_calls = None
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
                elif item.type == "image_url" and provider.get("image", True):
                    image_message = await get_image_message(item.image_url.url, engine)
                    content.append(image_message)
        elif msg.content:
            content = [{"text": msg.content}]
        elif msg.content is None:
            tool_calls = msg.tool_calls

        if tool_calls:
            tool_call = tool_calls[0]
            function_arguments = {
                "functionCall": {
                    "name": tool_call.function.name,
                    "args": json.loads(tool_call.function.arguments)
                }
            }
            messages.append(
                {
                    "role": "model",
                    "parts": [function_arguments]
                }
            )
        elif msg.role == "tool":
            function_call_name = function_arguments["functionCall"]["name"]
            messages.append(
                {
                    "role": "function",
                    "parts": [{
                    "functionResponse": {
                        "name": function_call_name,
                        "response": {
                            "name": function_call_name,
                            "content": {
                                "result": msg.content,
                            }
                        }
                    }
                    }]
                }
            )
        elif msg.role != "system" and content:
            messages.append({"role": msg.role, "parts": content})
        elif msg.role == "system":
            system_prompt = system_prompt + "\n\n" + content[0]["text"]
    if system_prompt.strip():
        systemInstruction = {"parts": [{"text": system_prompt}]}

    if any(off_model in original_model for off_model in gemini_max_token_65k_models):
        safety_settings = "OFF"
    else:
        safety_settings = "BLOCK_NONE"

    payload = {
        "contents": messages,
        "safetySettings": [
            {
                "category": "HARM_CATEGORY_HARASSMENT",
                "threshold": safety_settings
            },
            {
                "category": "HARM_CATEGORY_HATE_SPEECH",
                "threshold": safety_settings
            },
            {
                "category": "HARM_CATEGORY_SEXUALLY_EXPLICIT",
                "threshold": safety_settings
            },
            {
                "category": "HARM_CATEGORY_DANGEROUS_CONTENT",
                "threshold": safety_settings
            },
            {
                "category": "HARM_CATEGORY_CIVIC_INTEGRITY",
                "threshold": "BLOCK_NONE"
            },
        ]
    }
    if systemInstruction:
        payload["system_instruction"] = systemInstruction

    miss_fields = [
        'model',
        'messages',
        'stream',
        'tool_choice',
        'presence_penalty',
        'frequency_penalty',
        'n',
        'user',
        'include_usage',
        'logprobs',
        'top_logprobs',
        'stream_options',
        'prompt',
        'size',
    ]
    generation_config = {}

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            if field == "tools":
                payload.update({
                    "tools": [{
                        "function_declarations": [tool["function"] for tool in value]
                    }],
                    "tool_config": {
                        "function_calling_config": {
                            "mode": "AUTO"
                        }
                    }
                })
            elif field == "temperature":
                generation_config["temperature"] = value
            elif field == "max_tokens":
                if value > 65535:
                    value = 65535
                generation_config["max_output_tokens"] = value
            elif field == "top_p":
                generation_config["top_p"] = value
            else:
                payload[field] = value

    payload["generationConfig"] = generation_config
    if "max_output_tokens" not in generation_config:
        if any(pro_model in original_model for pro_model in gemini_max_token_65k_models):
            payload["generationConfig"]["max_output_tokens"] = 65535
        else:
            payload["generationConfig"]["max_output_tokens"] = 8192

    if "gemini-2.5" in original_model:
        # 从请求模型名中检测思考预算设置
        m = re.match(r".*-think-(-?\d+)", request.model)
        if m:
            try:
                val = int(m.group(1))
                budget = None
                # gemini-2.5-pro: [128, 32768]
                if "gemini-2.5-pro" in original_model:
                    if val < 128:
                        budget = 128
                    elif val > 32768:
                        budget = 32768
                    else: # 128 <= val <= 32768
                        budget = val

                # gemini-2.5-flash-lite: [0] or [512, 24576]
                elif "gemini-2.5-flash-lite" in original_model:
                    if val > 0 and val < 512:
                        budget = 512
                    elif val > 24576:
                        budget = 24576
                    else: # Includes 0 and valid range, and clamps invalid negatives
                        budget = val if val >= 0 else 0

                # gemini-2.5-flash (and other gemini-2.5 models as a fallback): [0, 24576]
                else:
                    if val > 24576:
                        budget = 24576
                    else: # Includes 0 and valid range, and clamps invalid negatives
                        budget = val if val >= 0 else 0

                payload["generationConfig"]["thinkingConfig"] = {
                    "includeThoughts": True if budget else False,
                    "thinkingBudget": budget
                }
            except ValueError:
                # 如果转换为整数失败，忽略思考预算设置
                pass
        else:
            payload["generationConfig"]["thinkingConfig"] = {
                "includeThoughts": True,
            }

    if safe_get(provider, "preferences", "post_body_parameter_overrides", default=None):
        for key, value in safe_get(provider, "preferences", "post_body_parameter_overrides", default={}).items():
            if key == request.model:
                for k, v in value.items():
                    payload[k] = v
            elif all(_model not in request.model.lower() for _model in ["gemini", "gpt", "claude"]):
                payload[key] = value

    return url, headers, payload

async def get_vertex_claude_payload(request, engine, provider, api_key=None):
    headers = {
        'Content-Type': 'application/json',
    }
    if provider.get("client_email") and provider.get("private_key"):
        access_token = await get_access_token(provider['client_email'], provider['private_key'])
        headers['Authorization'] = f"Bearer {access_token}"
    if provider.get("project_id"):
        project_id = provider.get("project_id")

    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    if "claude-3-5-sonnet" in original_model or "claude-3-7-sonnet" in original_model:
        location = c35s
    elif "claude-3-opus" in original_model:
        location = c3o
    elif "claude-sonnet-4" in original_model or "claude-opus-4" in original_model:
        location = c4
    elif "claude-3-sonnet" in original_model:
        location = c3s
    elif "claude-3-haiku" in original_model:
        location = c3h

    claude_stream = "streamRawPredict"
    url = "https://{LOCATION}-aiplatform.googleapis.com/v1/projects/{PROJECT_ID}/locations/{LOCATION}/publishers/anthropic/models/{MODEL}:{stream}".format(
        LOCATION=await location.next(),
        PROJECT_ID=project_id,
        MODEL=original_model,
        stream=claude_stream
    )

    messages = []
    system_prompt = None
    tool_id = None
    for msg in request.messages:
        tool_call_id = None
        tool_calls = None
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
                elif item.type == "image_url" and provider.get("image", True):
                    image_message = await get_image_message(item.image_url.url, engine)
                    content.append(image_message)
        else:
            content = msg.content
            tool_calls = msg.tool_calls
            tool_id = tool_calls[0].id if tool_calls else None or tool_id
            tool_call_id = msg.tool_call_id

        if tool_calls:
            tool_calls_list = []
            tool_call = tool_calls[0]
            tool_calls_list.append({
                "type": "tool_use",
                "id": tool_call.id,
                "name": tool_call.function.name,
                "input": json.loads(tool_call.function.arguments),
            })
            messages.append({"role": msg.role, "content": tool_calls_list})
        elif tool_call_id:
            messages.append({"role": "user", "content": [{
                "type": "tool_result",
                "tool_use_id": tool_id,
                "content": content
            }]})
        elif msg.role == "function":
            messages.append({"role": "assistant", "content": [{
                "type": "tool_use",
                "id": "toolu_017r5miPMV6PGSNKmhvHPic4",
                "name": msg.name,
                "input": {"prompt": "..."}
            }]})
            messages.append({"role": "user", "content": [{
                "type": "tool_result",
                "tool_use_id": "toolu_017r5miPMV6PGSNKmhvHPic4",
                "content": msg.content
            }]})
        elif msg.role != "system":
            messages.append({"role": msg.role, "content": content})
        elif msg.role == "system":
            system_prompt = content

    conversation_len = len(messages) - 1
    message_index = 0
    while message_index < conversation_len:
        if messages[message_index]["role"] == messages[message_index + 1]["role"]:
            if messages[message_index].get("content"):
                if isinstance(messages[message_index]["content"], list):
                    messages[message_index]["content"].extend(messages[message_index + 1]["content"])
                elif isinstance(messages[message_index]["content"], str) and isinstance(messages[message_index + 1]["content"], list):
                    content_list = [{"type": "text", "text": messages[message_index]["content"]}]
                    content_list.extend(messages[message_index + 1]["content"])
                    messages[message_index]["content"] = content_list
                else:
                    messages[message_index]["content"] += messages[message_index + 1]["content"]
            messages.pop(message_index + 1)
            conversation_len = conversation_len - 1
        else:
            message_index = message_index + 1

    if "claude-3-7-sonnet" in original_model:
        max_tokens = 20000
    elif "claude-3-5-sonnet" in original_model:
        max_tokens = 8192
    else:
        max_tokens = 4096

    payload = {
        "anthropic_version": "vertex-2023-10-16",
        "messages": messages,
        "system": system_prompt or "You are Claude, a large language model trained by Anthropic.",
        "max_tokens": max_tokens,
    }

    if request.max_tokens:
        payload["max_tokens"] = int(request.max_tokens)

    miss_fields = [
        'model',
        'messages',
        'presence_penalty',
        'frequency_penalty',
        'n',
        'user',
        'include_usage',
        'stream_options',
    ]

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            payload[field] = value

    if request.tools and provider.get("tools"):
        tools = []
        for tool in request.tools:
            json_tool = await gpt2claude_tools_json(tool.dict()["function"])
            tools.append(json_tool)
        payload["tools"] = tools
        if "tool_choice" in payload:
            if isinstance(payload["tool_choice"], dict):
                if payload["tool_choice"]["type"] == "function":
                    payload["tool_choice"] = {
                        "type": "tool",
                        "name": payload["tool_choice"]["function"]["name"]
                    }
            if isinstance(payload["tool_choice"], str):
                if payload["tool_choice"] == "auto":
                    payload["tool_choice"] = {
                        "type": "auto"
                    }
                if payload["tool_choice"] == "none":
                    payload["tool_choice"] = {
                        "type": "any"
                    }

    if provider.get("tools") == False:
        payload.pop("tools", None)
        payload.pop("tool_choice", None)

    return url, headers, payload

import hashlib
import hmac
import datetime
import urllib.parse
from datetime import timezone

def sign(key, msg):
    return hmac.new(key, msg.encode('utf-8'), hashlib.sha256).digest()

def get_signature_key(key, date_stamp, region_name, service_name):
    k_date = sign(('AWS4' + key).encode('utf-8'), date_stamp)
    k_region = sign(k_date, region_name)
    k_service = sign(k_region, service_name)
    k_signing = sign(k_service, 'aws4_request')
    return k_signing

def get_signature(request_body, model_id, aws_access_key, aws_secret_key, aws_region, host, content_type, accept_header):
    request_body = json.dumps(request_body)
    SERVICE = "bedrock"
    canonical_querystring = ''
    method = 'POST'
    raw_path = f'/model/{model_id}/invoke-with-response-stream'
    canonical_uri = urllib.parse.quote(raw_path, safe='/-_.~')
    # Create a date for headers and the credential string
    t = datetime.datetime.now(timezone.utc)
    amz_date = t.strftime('%Y%m%dT%H%M%SZ')
    date_stamp = t.strftime('%Y%m%d') # Date YYYYMMDD

    # --- Task 1: Create a Canonical Request ---
    payload_hash = hashlib.sha256(request_body.encode('utf-8')).hexdigest()

    canonical_headers = f'accept:{accept_header}\n' \
                        f'content-type:{content_type}\n' \
                        f'host:{host}\n' \
                        f'x-amz-bedrock-accept:{accept_header}\n' \
                        f'x-amz-content-sha256:{payload_hash}\n' \
                        f'x-amz-date:{amz_date}\n'
    # 注意：头名称需要按字母顺序排序

    signed_headers = 'accept;content-type;host;x-amz-bedrock-accept;x-amz-content-sha256;x-amz-date' # 按字母顺序排序

    canonical_request = f'{method}\n' \
                        f'{canonical_uri}\n' \
                        f'{canonical_querystring}\n' \
                        f'{canonical_headers}\n' \
                        f'{signed_headers}\n' \
                        f'{payload_hash}'

    # --- Task 2: Create the String to Sign ---
    algorithm = 'AWS4-HMAC-SHA256'
    credential_scope = f'{date_stamp}/{aws_region}/{SERVICE}/aws4_request'
    string_to_sign = f'{algorithm}\n' \
                    f'{amz_date}\n' \
                    f'{credential_scope}\n' \
                    f'{hashlib.sha256(canonical_request.encode("utf-8")).hexdigest()}'

    # --- Task 3: Calculate the Signature ---
    signing_key = get_signature_key(aws_secret_key, date_stamp, aws_region, SERVICE)
    signature = hmac.new(signing_key, string_to_sign.encode('utf-8'), hashlib.sha256).hexdigest()

    # --- Task 4: Add Signing Information to the Request ---
    authorization_header = f'{algorithm} Credential={aws_access_key}/{credential_scope}, SignedHeaders={signed_headers}, Signature={signature}'
    return amz_date, payload_hash, authorization_header

async def get_aws_payload(request, engine, provider, api_key=None):
    CONTENT_TYPE = "application/json"
    # AWS_REGION = "us-east-1"
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    # MODEL_ID = "anthropic.claude-3-5-sonnet-20240620-v1:0"
    base_url = provider.get('base_url')
    AWS_REGION = base_url.split('.')[1]
    HOST = f"bedrock-runtime.{AWS_REGION}.amazonaws.com"
    # url = f"{base_url}/model/{original_model}/invoke"
    url = f"{base_url}/model/{original_model}/invoke-with-response-stream"

    messages = []
    system_prompt = None
    tool_id = None
    for msg in request.messages:
        tool_call_id = None
        tool_calls = None
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
                elif item.type == "image_url" and provider.get("image", True):
                    image_message = await get_image_message(item.image_url.url, engine)
                    content.append(image_message)
        else:
            content = msg.content
            tool_calls = msg.tool_calls
            tool_id = tool_calls[0].id if tool_calls else None or tool_id
            tool_call_id = msg.tool_call_id

        if tool_calls:
            tool_calls_list = []
            tool_call = tool_calls[0]
            tool_calls_list.append({
                "type": "tool_use",
                "id": tool_call.id,
                "name": tool_call.function.name,
                "input": json.loads(tool_call.function.arguments),
            })
            messages.append({"role": msg.role, "content": tool_calls_list})
        elif tool_call_id:
            messages.append({"role": "user", "content": [{
                "type": "tool_result",
                "tool_use_id": tool_id,
                "content": content
            }]})
        elif msg.role == "function":
            messages.append({"role": "assistant", "content": [{
                "type": "tool_use",
                "id": "toolu_017r5miPMV6PGSNKmhvHPic4",
                "name": msg.name,
                "input": {"prompt": "..."}
            }]})
            messages.append({"role": "user", "content": [{
                "type": "tool_result",
                "tool_use_id": "toolu_017r5miPMV6PGSNKmhvHPic4",
                "content": msg.content
            }]})
        elif msg.role != "system":
            messages.append({"role": msg.role, "content": content})
        elif msg.role == "system":
            system_prompt = content

    conversation_len = len(messages) - 1
    message_index = 0
    while message_index < conversation_len:
        if messages[message_index]["role"] == messages[message_index + 1]["role"]:
            if messages[message_index].get("content"):
                if isinstance(messages[message_index]["content"], list):
                    messages[message_index]["content"].extend(messages[message_index + 1]["content"])
                elif isinstance(messages[message_index]["content"], str) and isinstance(messages[message_index + 1]["content"], list):
                    content_list = [{"type": "text", "text": messages[message_index]["content"]}]
                    content_list.extend(messages[message_index + 1]["content"])
                    messages[message_index]["content"] = content_list
                else:
                    messages[message_index]["content"] += messages[message_index + 1]["content"]
            messages.pop(message_index + 1)
            conversation_len = conversation_len - 1
        else:
            message_index = message_index + 1

    # if "claude-3-7-sonnet" in original_model:
    #     max_tokens = 20000
    # elif "claude-3-5-sonnet" in original_model:
    #     max_tokens = 8192
    # else:
    #     max_tokens = 4096
    max_tokens = 4096

    payload = {
        "messages": messages,
        "anthropic_version": "bedrock-2023-05-31",
        "max_tokens": max_tokens,
    }

    # payload = {
    #     "anthropic_version": "vertex-2023-10-16",
    #     "messages": messages,
    #     "system": system_prompt or "You are Claude, a large language model trained by Anthropic.",
    #     "max_tokens": max_tokens,
    # }

    if request.max_tokens:
        payload["max_tokens"] = int(request.max_tokens)

    miss_fields = [
        'model',
        'messages',
        'presence_penalty',
        'frequency_penalty',
        'n',
        'user',
        'include_usage',
        'stream_options',
        'stream',
    ]

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            payload[field] = value

    if request.tools and provider.get("tools"):
        tools = []
        for tool in request.tools:
            json_tool = await gpt2claude_tools_json(tool.dict()["function"])
            tools.append(json_tool)
        payload["tools"] = tools
        if "tool_choice" in payload:
            if isinstance(payload["tool_choice"], dict):
                if payload["tool_choice"]["type"] == "function":
                    payload["tool_choice"] = {
                        "type": "tool",
                        "name": payload["tool_choice"]["function"]["name"]
                    }
            if isinstance(payload["tool_choice"], str):
                if payload["tool_choice"] == "auto":
                    payload["tool_choice"] = {
                        "type": "auto"
                    }
                if payload["tool_choice"] == "none":
                    payload["tool_choice"] = {
                        "type": "any"
                    }

    if provider.get("tools") == False:
        payload.pop("tools", None)
        payload.pop("tool_choice", None)

    if provider.get("aws_access_key") and provider.get("aws_secret_key"):
        ACCEPT_HEADER = "application/vnd.amazon.bedrock.payload+json" # 指定接受 Bedrock 流格式
        amz_date, payload_hash, authorization_header = await asyncio.to_thread(
            get_signature, payload, original_model, provider.get("aws_access_key"), provider.get("aws_secret_key"), AWS_REGION, HOST, CONTENT_TYPE, ACCEPT_HEADER
        )
        headers = {
            'Accept': ACCEPT_HEADER,
            'Content-Type': CONTENT_TYPE,
            'X-Amz-Date': amz_date,
            'X-Amz-Bedrock-Accept': ACCEPT_HEADER, # Bedrock 特定头
            'X-Amz-Content-Sha256': payload_hash,
            'Authorization': authorization_header,
            # Add 'X-Amz-Security-Token': SESSION_TOKEN if using temporary credentials
        }

    return url, headers, payload

async def get_gpt_payload(request, engine, provider, api_key=None):
    headers = {
        'Content-Type': 'application/json',
    }
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    if api_key:
        headers['Authorization'] = f"Bearer {api_key}"

    url = provider['base_url']
    if "openrouter.ai" in url:
        headers['HTTP-Referer'] = "https://github.com/yym68686/uni-api"
        headers['X-Title'] = "Uni API"

    messages = []
    for msg in request.messages:
        tool_calls = None
        tool_call_id = None
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
                elif item.type == "image_url" and provider.get("image", True) and "o1-mini" not in original_model:
                    image_message = await get_image_message(item.image_url.url, engine)
                    content.append(image_message)
        else:
            content = msg.content
            if msg.role == "system" and "o3-mini" in original_model and not content.startswith("Formatting re-enabled"):
                content = "Formatting re-enabled. " + content
            tool_calls = msg.tool_calls
            tool_call_id = msg.tool_call_id

        if tool_calls:
            tool_calls_list = []
            for tool_call in tool_calls:
                tool_calls_list.append({
                    "id": tool_call.id,
                    "type": tool_call.type,
                    "function": {
                        "name": tool_call.function.name,
                        "arguments": tool_call.function.arguments
                    }
                })
                if provider.get("tools"):
                    messages.append({"role": msg.role, "tool_calls": tool_calls_list})
        elif tool_call_id:
            if provider.get("tools"):
                messages.append({"role": msg.role, "tool_call_id": tool_call_id, "content": content})
        else:
            messages.append({"role": msg.role, "content": content})

    if ("o1-mini" in original_model or "o1-preview" in original_model) and len(messages) > 1 and messages[0]["role"] == "system":
        system_msg = messages.pop(0)
        messages[0]["content"] = system_msg["content"] + messages[0]["content"]

    payload = {
        "model": original_model,
        "messages": messages,
    }

    miss_fields = [
        'model',
        'messages',
    ]

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            if field == "max_tokens" and ("o1" in original_model or "o3" in original_model or "o4" in original_model or "gpt-5" in original_model):
                payload["max_completion_tokens"] = value
            else:
                payload[field] = value

    if provider.get("tools") == False or "o1-mini" in original_model or "chatgpt-4o-latest" in original_model or "grok" in original_model:
        payload.pop("tools", None)
        payload.pop("tool_choice", None)

    if "api.x.ai" in url:
        payload.pop("stream_options", None)
        payload.pop("presence_penalty", None)
        payload.pop("frequency_penalty", None)

    if "grok-3-mini" in original_model:
        if request.model.endswith("high"):
            payload["reasoning_effort"] = "high"
        elif request.model.endswith("low"):
            payload["reasoning_effort"] = "low"

    if "o1" in original_model or \
    "o3" in original_model or "o4" in original_model or \
    "gpt-oss" in original_model or "gpt-5" in original_model:
        if request.model.endswith("high"):
            payload["reasoning_effort"] = "high"
        elif request.model.endswith("low"):
            payload["reasoning_effort"] = "low"
        else:
            payload["reasoning_effort"] = "medium"

        if "temperature" in payload:
            payload.pop("temperature")

    # 代码生成/数学解题  0.0
    # 数据抽取/分析	     1.0
    # 通用对话          1.3
    # 翻译	           1.3
    # 创意类写作/诗歌创作 1.5
    if "deepseek-r" in original_model.lower():
        if "temperature" not in payload:
            payload["temperature"] = 0.6

    if request.model.endswith("-search") and "gemini" in original_model:
        if "tools" not in payload:
            payload["tools"] = [{
                "type": "function",
                "function": {
                    "name": "googleSearch",
                    "description": "googleSearch"
                }
            }]
        else:
            if not any(tool["function"]["name"] == "googleSearch" for tool in payload["tools"]):
                payload["tools"].append({
                    "type": "function",
                    "function": {
                        "name": "googleSearch",
                        "description": "googleSearch"
                    }
                })

    if safe_get(provider, "preferences", "post_body_parameter_overrides", default=None):
        for key, value in safe_get(provider, "preferences", "post_body_parameter_overrides", default={}).items():
            if key == request.model:
                for k, v in value.items():
                    payload[k] = v
            elif all(_model not in request.model.lower() for _model in ["gemini", "gpt", "claude"]):
                payload[key] = value

    return url, headers, payload

def build_azure_endpoint(base_url, deployment_id, api_version="2025-01-01-preview"):
    # 移除base_url末尾的斜杠(如果有)
    base_url = base_url.rstrip('/')
    final_url = base_url

    if "models/chat/completions" not in final_url:
        # 构建路径
        path = f"/openai/deployments/{deployment_id}/chat/completions"
        # 使用urljoin拼接base_url和path
        final_url = urllib.parse.urljoin(base_url, path)

    if "?api-version=" not in final_url:
        # 添加api-version查询参数
        final_url = f"{final_url}?api-version={api_version}"

    return final_url

async def get_azure_payload(request, engine, provider, api_key=None):
    headers = {
        'Content-Type': 'application/json',
    }
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    headers['api-key'] = f"{api_key}"

    url = build_azure_endpoint(
        base_url=provider['base_url'],
        deployment_id=original_model,
    )

    messages = []
    for msg in request.messages:
        tool_calls = None
        tool_call_id = None
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
                elif item.type == "image_url" and provider.get("image", True) and "o1-mini" not in original_model:
                    image_message = await get_image_message(item.image_url.url, engine)
                    content.append(image_message)
        else:
            content = msg.content
            tool_calls = msg.tool_calls
            tool_call_id = msg.tool_call_id

        if tool_calls:
            tool_calls_list = []
            for tool_call in tool_calls:
                tool_calls_list.append({
                    "id": tool_call.id,
                    "type": tool_call.type,
                    "function": {
                        "name": tool_call.function.name,
                        "arguments": tool_call.function.arguments
                    }
                })
                if provider.get("tools"):
                    messages.append({"role": msg.role, "tool_calls": tool_calls_list})
        elif tool_call_id:
            if provider.get("tools"):
                messages.append({"role": msg.role, "tool_call_id": tool_call_id, "content": content})
        else:
            messages.append({"role": msg.role, "content": content})

    payload = {
        "model": original_model,
        "messages": messages,
    }

    miss_fields = [
        'model',
        'messages',
    ]

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            if field == "max_tokens" and "o1" in original_model:
                payload["max_completion_tokens"] = value
            else:
                payload[field] = value

    if provider.get("tools") == False or "o1" in original_model or "chatgpt-4o-latest" in original_model or "grok" in original_model:
        payload.pop("tools", None)
        payload.pop("tool_choice", None)

    if safe_get(provider, "preferences", "post_body_parameter_overrides", default=None):
        for key, value in safe_get(provider, "preferences", "post_body_parameter_overrides", default={}).items():
            if key == request.model:
                for k, v in value.items():
                    payload[k] = v
            elif all(_model not in request.model.lower() for _model in ["gemini", "gpt", "claude"]):
                payload[key] = value

    return url, headers, payload

async def get_azure_databricks_payload(request, engine, provider, api_key=None):
    api_key = base64.b64encode(f"token:{api_key}".encode()).decode()
    headers = {
        'Content-Type': 'application/json',
        'Authorization': f"Basic {api_key}",
    }
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]

    base_url=provider['base_url']
    url = urllib.parse.urljoin(base_url, f"/serving-endpoints/{original_model}/invocations")

    messages = []
    for msg in request.messages:
        tool_calls = None
        tool_call_id = None
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
                elif item.type == "image_url" and provider.get("image", True) and "o1-mini" not in original_model:
                    image_message = await get_image_message(item.image_url.url, engine)
                    content.append(image_message)
        else:
            content = msg.content
            tool_calls = msg.tool_calls
            tool_call_id = msg.tool_call_id

        if tool_calls:
            tool_calls_list = []
            for tool_call in tool_calls:
                tool_calls_list.append({
                    "id": tool_call.id,
                    "type": tool_call.type,
                    "function": {
                        "name": tool_call.function.name,
                        "arguments": tool_call.function.arguments
                    }
                })
                if provider.get("tools"):
                    messages.append({"role": msg.role, "tool_calls": tool_calls_list})
        elif tool_call_id:
            if provider.get("tools"):
                messages.append({"role": msg.role, "tool_call_id": tool_call_id, "content": content})
        else:
            messages.append({"role": msg.role, "content": content})

    if "claude-3-7-sonnet" in original_model:
        max_tokens = 128000
    elif "claude-3-5-sonnet" in original_model:
        max_tokens = 8192
    elif "claude-sonnet-4" in original_model:
        max_tokens = 64000
    elif "claude-opus-4" in original_model:
        max_tokens = 32000
    else:
        max_tokens = 4096

    payload = {
        "model": original_model,
        "messages": messages,
        "max_tokens": max_tokens,
    }

    if request.max_tokens:
        payload["max_tokens"] = int(request.max_tokens)

    miss_fields = [
        'model',
        'messages',
    ]

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            if field == "max_tokens" and "o1" in original_model:
                payload["max_completion_tokens"] = value
            else:
                payload[field] = value

    if provider.get("tools") == False or "o1" in original_model or "chatgpt-4o-latest" in original_model or "grok" in original_model:
        payload.pop("tools", None)
        payload.pop("tool_choice", None)

    if "think" in request.model.lower():
        payload["thinking"] = {
            "budget_tokens": 4096,
            "type": "enabled"
        }
        payload["temperature"] = 1
        payload.pop("top_p", None)
        payload.pop("top_k", None)
        if request.model.split("-")[-1].isdigit():
            think_tokens = int(request.model.split("-")[-1])
            if think_tokens < max_tokens:
                payload["thinking"] = {
                    "budget_tokens": think_tokens,
                    "type": "enabled"
                }

    if request.thinking:
        payload["thinking"] = {
            "budget_tokens": request.thinking.budget_tokens,
            "type": request.thinking.type
        }
        payload["temperature"] = 1
        payload.pop("top_p", None)
        payload.pop("top_k", None)

    if safe_get(provider, "preferences", "post_body_parameter_overrides", default=None):
        for key, value in safe_get(provider, "preferences", "post_body_parameter_overrides", default={}).items():
            if key == request.model:
                for k, v in value.items():
                    payload[k] = v
            elif all(_model not in request.model.lower() for _model in ["gemini", "gpt", "claude"]):
                payload[key] = value

    return url, headers, payload

async def get_openrouter_payload(request, engine, provider, api_key=None):
    headers = {
        'Content-Type': 'application/json'
    }
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    if api_key:
        headers['Authorization'] = f"Bearer {api_key}"

    url = provider['base_url']
    if "openrouter.ai" in url:
        headers['HTTP-Referer'] = "https://github.com/yym68686/uni-api"
        headers['X-Title'] = "Uni API"

    messages = []
    for msg in request.messages:
        tool_calls = None
        tool_call_id = None
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
                elif item.type == "image_url" and provider.get("image", True):
                    image_message = await get_image_message(item.image_url.url, engine)
                    content.append(image_message)
        else:
            content = msg.content
            tool_calls = msg.tool_calls
            tool_call_id = msg.tool_call_id

        if tool_calls:
            tool_calls_list = []
            for tool_call in tool_calls:
                tool_calls_list.append({
                    "id": tool_call.id,
                    "type": tool_call.type,
                    "function": {
                        "name": tool_call.function.name,
                        "arguments": tool_call.function.arguments
                    }
                })
                if provider.get("tools"):
                    messages.append({"role": msg.role, "tool_calls": tool_calls_list})
        elif tool_call_id:
            if provider.get("tools"):
                messages.append({"role": msg.role, "tool_call_id": tool_call_id, "content": content})
        else:
            # print("content", content)
            if isinstance(content, list):
                for item in content:
                    if item["type"] == "text":
                        messages.append({"role": msg.role, "content": item["text"]})
                    elif item["type"] == "image_url":
                        messages.append({"role": msg.role, "content": [await get_image_message(item["image_url"]["url"], engine)]})
            else:
                messages.append({"role": msg.role, "content": content})

    payload = {
        "model": original_model,
        "messages": messages,
    }

    miss_fields = [
        'model',
        'messages',
        'n',
        'user',
        'include_usage',
        'stream_options',
    ]

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            payload[field] = value

    if safe_get(provider, "preferences", "post_body_parameter_overrides", default=None):
        for key, value in safe_get(provider, "preferences", "post_body_parameter_overrides", default={}).items():
            if key == request.model:
                for k, v in value.items():
                    payload[k] = v
            elif all(_model not in request.model.lower() for _model in ["gemini", "gpt", "claude"]):
                payload[key] = value

    return url, headers, payload

async def get_cohere_payload(request, engine, provider, api_key=None):
    headers = {
        'Content-Type': 'application/json'
    }
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    if api_key:
        headers['Authorization'] = f"Bearer {api_key}"

    url = provider['base_url']

    role_map = {
        "user": "USER",
        "assistant" : "CHATBOT",
        "system": "SYSTEM"
    }

    messages = []
    for msg in request.messages:
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
        else:
            content = msg.content

        if isinstance(content, list):
            for item in content:
                if item["type"] == "text":
                    messages.append({"role": role_map[msg.role], "message": item["text"]})
        else:
            messages.append({"role": role_map[msg.role], "message": content})

    chat_history = messages[:-1]
    query = messages[-1].get("message")
    payload = {
        "model": original_model,
        "message": query,
    }

    if chat_history:
        payload["chat_history"] = chat_history

    miss_fields = [
        'model',
        'messages',
        'tools',
        'tool_choice',
        'temperature',
        'top_p',
        'max_tokens',
        'presence_penalty',
        'frequency_penalty',
        'n',
        'user',
        'include_usage',
        'logprobs',
        'top_logprobs',
        'stream_options',
    ]

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            payload[field] = value

    return url, headers, payload

async def get_cloudflare_payload(request, engine, provider, api_key=None):
    headers = {
        'Content-Type': 'application/json'
    }
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    if api_key:
        headers['Authorization'] = f"Bearer {api_key}"

    url = "https://api.cloudflare.com/client/v4/accounts/{cf_account_id}/ai/run/{cf_model_id}".format(cf_account_id=provider['cf_account_id'], cf_model_id=original_model)

    msg = request.messages[-1]
    content = None
    if isinstance(msg.content, list):
        for item in msg.content:
            if item.type == "text":
                content = await get_text_message(item.text, engine)
    else:
        content = msg.content

    payload = {
        "prompt": content,
    }

    miss_fields = [
        'model',
        'messages',
        'tools',
        'tool_choice',
        'temperature',
        'top_p',
        'max_tokens',
        'presence_penalty',
        'frequency_penalty',
        'n',
        'user',
        'include_usage',
        'logprobs',
        'top_logprobs',
        'stream_options',
    ]

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            payload[field] = value

    return url, headers, payload

async def gpt2claude_tools_json(json_dict):
    import copy
    json_dict = copy.deepcopy(json_dict)

    # 处理 $ref 引用
    def resolve_refs(obj, defs):
        if isinstance(obj, dict):
            # 如果有 $ref 引用，替换为实际定义
            if "$ref" in obj and obj["$ref"].startswith("#/$defs/"):
                ref_name = obj["$ref"].split("/")[-1]
                if ref_name in defs:
                    # 完全替换为引用的对象
                    ref_obj = copy.deepcopy(defs[ref_name])
                    # 保留原始对象中的其他属性
                    for k, v in obj.items():
                        if k != "$ref":
                            ref_obj[k] = v
                    return ref_obj

            # 递归处理所有属性
            for key, value in list(obj.items()):
                obj[key] = resolve_refs(value, defs)

        elif isinstance(obj, list):
            # 递归处理列表中的每个元素
            for i, item in enumerate(obj):
                obj[i] = resolve_refs(item, defs)

        return obj

    # 提取 $defs 定义
    defs = {}
    if "parameters" in json_dict and "defs" in json_dict["parameters"]:
        defs = json_dict["parameters"]["defs"]
        # 从参数中删除 $defs，因为 Claude 不需要它
        del json_dict["parameters"]["defs"]

    # 解析所有引用
    json_dict = resolve_refs(json_dict, defs)

    # 继续原有的键名转换逻辑
    keys_to_change = {
        "parameters": "input_schema",
    }
    for old_key, new_key in keys_to_change.items():
        if old_key in json_dict:
            if new_key:
                if json_dict[old_key] == None:
                    json_dict[old_key] = {
                        "type": "object",
                        "properties": {}
                    }
                json_dict[new_key] = json_dict.pop(old_key)
            else:
                json_dict.pop(old_key)
    return json_dict

async def get_claude_payload(request, engine, provider, api_key=None):
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]

    if "claude-3-7-sonnet" in original_model:
        anthropic_beta = "output-128k-2025-02-19"
    elif "claude-3-5-sonnet" in original_model:
        anthropic_beta = "max-tokens-3-5-sonnet-2024-07-15"
    else:
        anthropic_beta = "tools-2024-05-16"

    headers = {
        "content-type": "application/json",
        "x-api-key": f"{api_key}",
        "anthropic-version": "2023-06-01",
        "anthropic-beta": anthropic_beta,
    }
    url = provider['base_url']

    messages = []
    system_prompt = None
    tool_id = None
    for msg in request.messages:
        tool_call_id = None
        tool_calls = None
        if isinstance(msg.content, list):
            content = []
            for item in msg.content:
                if item.type == "text":
                    text_message = await get_text_message(item.text, engine)
                    content.append(text_message)
                elif item.type == "image_url" and provider.get("image", True):
                    image_message = await get_image_message(item.image_url.url, engine)
                    content.append(image_message)
        else:
            content = msg.content
            tool_calls = msg.tool_calls
            tool_id = tool_calls[0].id if tool_calls else None or tool_id
            tool_call_id = msg.tool_call_id

        if tool_calls:
            tool_calls_list = []
            tool_call = tool_calls[0]
            tool_calls_list.append({
                "type": "tool_use",
                "id": tool_call.id,
                "name": tool_call.function.name,
                "input": json.loads(tool_call.function.arguments),
            })
            messages.append({"role": msg.role, "content": tool_calls_list})
        elif tool_call_id:
            messages.append({"role": "user", "content": [{
                "type": "tool_result",
                "tool_use_id": tool_id,
                "content": content
            }]})
        elif msg.role == "function":
            messages.append({"role": "assistant", "content": [{
                "type": "tool_use",
                "id": "toolu_017r5miPMV6PGSNKmhvHPic4",
                "name": msg.name,
                "input": {"prompt": "..."}
            }]})
            messages.append({"role": "user", "content": [{
                "type": "tool_result",
                "tool_use_id": "toolu_017r5miPMV6PGSNKmhvHPic4",
                "content": msg.content
            }]})
        elif msg.role != "system":
            messages.append({"role": msg.role, "content": content})
        elif msg.role == "system":
            system_prompt = content

    conversation_len = len(messages) - 1
    message_index = 0
    while message_index < conversation_len:
        if messages[message_index]["role"] == messages[message_index + 1]["role"]:
            if messages[message_index].get("content"):
                if isinstance(messages[message_index]["content"], list):
                    messages[message_index]["content"].extend(messages[message_index + 1]["content"])
                elif isinstance(messages[message_index]["content"], str) and isinstance(messages[message_index + 1]["content"], list):
                    content_list = [{"type": "text", "text": messages[message_index]["content"]}]
                    content_list.extend(messages[message_index + 1]["content"])
                    messages[message_index]["content"] = content_list
                else:
                    messages[message_index]["content"] += messages[message_index + 1]["content"]
            messages.pop(message_index + 1)
            conversation_len = conversation_len - 1
        else:
            message_index = message_index + 1

    if "claude-3-7-sonnet" in original_model:
        max_tokens = 128000
    elif "claude-3-5-sonnet" in original_model:
        max_tokens = 8192
    elif "claude-sonnet-4" in original_model:
        max_tokens = 64000
    elif "claude-opus-4" in original_model:
        max_tokens = 32000
    else:
        max_tokens = 4096

    payload = {
        "model": original_model,
        "messages": messages,
        "max_tokens": max_tokens,
    }
    if system_prompt:
        payload["system"] = system_prompt

    if request.max_tokens:
        payload["max_tokens"] = int(request.max_tokens)

    miss_fields = [
        'model',
        'messages',
        'presence_penalty',
        'frequency_penalty',
        'n',
        'user',
        'include_usage',
        'stream_options',
    ]

    for field, value in request.model_dump(exclude_unset=True).items():
        if field not in miss_fields and value is not None:
            payload[field] = value

    if request.tools and provider.get("tools"):
        tools = []
        for tool in request.tools:
            # print("tool", type(tool), tool)
            json_tool = await gpt2claude_tools_json(tool.dict()["function"])
            tools.append(json_tool)
        payload["tools"] = tools
        if "tool_choice" in payload:
            if isinstance(payload["tool_choice"], dict):
                if payload["tool_choice"]["type"] == "function":
                    payload["tool_choice"] = {
                        "type": "tool",
                        "name": payload["tool_choice"]["function"]["name"]
                    }
            if isinstance(payload["tool_choice"], str):
                if payload["tool_choice"] == "auto":
                    payload["tool_choice"] = {
                        "type": "auto"
                    }
                if payload["tool_choice"] == "none":
                    payload["tool_choice"] = {
                        "type": "any"
                    }

    if provider.get("tools") == False:
        payload.pop("tools", None)
        payload.pop("tool_choice", None)

    if "think" in request.model.lower():
        payload["thinking"] = {
            "budget_tokens": 4096,
            "type": "enabled"
        }
        payload["temperature"] = 1
        payload.pop("top_p", None)
        payload.pop("top_k", None)
        if request.model.split("-")[-1].isdigit():
            think_tokens = int(request.model.split("-")[-1])
            if think_tokens < max_tokens:
                payload["thinking"] = {
                    "budget_tokens": think_tokens,
                    "type": "enabled"
                }

    if request.thinking:
        payload["thinking"] = {
            "budget_tokens": request.thinking.budget_tokens,
            "type": request.thinking.type
        }
        payload["temperature"] = 1
        payload.pop("top_p", None)
        payload.pop("top_k", None)
    # print("payload", json.dumps(payload, indent=2, ensure_ascii=False))

    if safe_get(provider, "preferences", "post_body_parameter_overrides", default=None):
        for key, value in safe_get(provider, "preferences", "post_body_parameter_overrides", default={}).items():
            if key == request.model:
                for k, v in value.items():
                    payload[k] = v
            elif all(_model not in request.model.lower() for _model in ["gemini", "gpt", "claude"]):
                payload[key] = value

    return url, headers, payload

async def get_dalle_payload(request, engine, provider, api_key=None):
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    headers = {
        "Content-Type": "application/json",
    }
    if api_key:
        headers['Authorization'] = f"Bearer {api_key}"
    url = provider['base_url']
    url = BaseAPI(url).image_url

    payload = {
        "model": original_model,
        "prompt": request.prompt,
        "n": request.n,
        "response_format": request.response_format,
        "size": request.size
    }

    return url, headers, payload

async def get_upload_certificate(client: httpx.AsyncClient, api_key: str, model: str) -> dict:
    """第一步：获取文件上传凭证"""
    # print("步骤 1: 正在获取上传凭证...")
    headers = {"Authorization": f"Bearer {api_key}"}
    params = {"action": "getPolicy", "model": model}
    try:
        response = await client.get("https://dashscope.aliyuncs.com/api/v1/uploads", headers=headers, params=params)
        response.raise_for_status()  # 如果请求失败则抛出异常
        cert_data = response.json()
        # print("凭证获取成功。")
        return cert_data.get("data")
    except httpx.HTTPStatusError as e:
        print(f"获取凭证失败: HTTP {e.response.status_code}")
        print(f"响应内容: {e.response.text}")
        return None
    except Exception as e:
        print(f"获取凭证时发生未知错误: {e}")
        return None

from mimetypes import guess_type

async def upload_file_to_oss(client: httpx.AsyncClient, certificate: dict, file: Tuple[str, IOBase, str]) -> str:
    """第二步：使用凭证将文件内容上传到OSS"""
    upload_host = certificate.get("upload_host")
    upload_dir = certificate.get("upload_dir")
    object_key = f"{upload_dir}/{file[0]}"

    form_data = {
        "key": object_key,
        "policy": certificate.get("policy"),
        "OSSAccessKeyId": certificate.get("oss_access_key_id"),
        "signature": certificate.get("signature"),
        "success_action_status": "200",
        "x-oss-object-acl": certificate.get("x_oss_object_acl"),
        "x-oss-forbid-overwrite": certificate.get("x_oss_forbid_overwrite"),
    }

    files = {"file": file}

    try:
        response = await client.post(upload_host, data=form_data, files=files, timeout=3600)
        response.raise_for_status()
        # print("文件上传成功！")
        oss_url = f"oss://{object_key}"
        # print(f"文件OSS URL: {oss_url}")
        return oss_url
    except httpx.HTTPStatusError as e:
        print(f"上传文件失败: HTTP {e.response.status_code}")
        print(f"响应内容: {e.response.text}")
        return None
    except Exception as e:
        print(f"上传文件时发生未知错误: {e}")
        return None

async def get_whisper_payload(request, engine, provider, api_key=None):
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    headers = {}
    if api_key:
        headers['Authorization'] = f"Bearer {api_key}"
    url = provider['base_url']
    url = BaseAPI(url).audio_transcriptions

    if "dashscope.aliyuncs.com" in url:
        client = httpx.AsyncClient()
        certificate = await get_upload_certificate(client, api_key, original_model)
        if not certificate:
            return

        # 步骤 2: 上传文件
        oss_url = await upload_file_to_oss(client, certificate, request.file)
        headers = {
            "Authorization": f"Bearer {api_key}",
            "Content-Type": "application/json",
            "X-DashScope-OssResourceResolve": "enable"
        }
        payload = {
            "model": original_model,
            "input": {
                "messages": [
                    {
                        "role": "user",
                        "content": [{"audio": oss_url}]
                    }
                ]
            }
        }
    else:
        payload = {
            "model": original_model,
            "file": request.file,
        }

    if request.prompt:
        payload["prompt"] = request.prompt
    if request.response_format:
        payload["response_format"] = request.response_format
    if request.temperature:
        payload["temperature"] = request.temperature
    if request.language:
        payload["language"] = request.language

    # https://platform.openai.com/docs/api-reference/audio/createTranscription
    if request.timestamp_granularities:
        payload["timestamp_granularities[]"] = request.timestamp_granularities

    return url, headers, payload

async def get_moderation_payload(request, engine, provider, api_key=None):
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    headers = {
        "Content-Type": "application/json",
    }
    if api_key:
        headers['Authorization'] = f"Bearer {api_key}"
    url = provider['base_url']
    url = BaseAPI(url).moderations

    payload = {
        "model": original_model,
        "input": request.input,
    }

    return url, headers, payload

async def get_embedding_payload(request, engine, provider, api_key=None):
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    headers = {
        "Content-Type": "application/json",
    }
    if api_key:
        headers['Authorization'] = f"Bearer {api_key}"

    url = provider['base_url']
    url = BaseAPI(url).embeddings
    payload = {
        "input": request.input,
        "model": original_model,
    }

    if request.encoding_format:
        if url.startswith("https://api.jina.ai"):
            payload["embedding_type"] = request.encoding_format
        else:
            payload["encoding_format"] = request.encoding_format

    return url, headers, payload

async def get_tts_payload(request, engine, provider, api_key=None):
    model_dict = get_model_dict(provider)
    original_model = model_dict[request.model]
    headers = {
        "Content-Type": "application/json",
    }
    if api_key:
        headers['Authorization'] = f"Bearer {api_key}"
    url = provider['base_url']
    url = BaseAPI(url).audio_speech

    if "api.minimaxi.com" in url:
        payload = {
            "model": original_model,
            "text": request.input,
            "voice_setting": {
                "voice_id": request.voice
            }
        }
    else:
        payload = {
            "model": original_model,
            "input": request.input,
            "voice": request.voice,
        }

    if request.response_format:
        payload["response_format"] = request.response_format
    if request.speed:
        payload["speed"] = request.speed
    if request.stream is not None:
        payload["stream"] = request.stream

    return url, headers, payload


async def get_payload(request: RequestModel, engine, provider, api_key=None):
    if engine == "gemini":
        return await get_gemini_payload(request, engine, provider, api_key)
    elif engine == "vertex-gemini":
        return await get_vertex_gemini_payload(request, engine, provider, api_key)
    elif engine == "aws":
        return await get_aws_payload(request, engine, provider, api_key)
    elif engine == "vertex-claude":
        return await get_vertex_claude_payload(request, engine, provider, api_key)
    elif engine == "azure":
        return await get_azure_payload(request, engine, provider, api_key)
    elif engine == "azure-databricks":
        return await get_azure_databricks_payload(request, engine, provider, api_key)
    elif engine == "claude":
        return await get_claude_payload(request, engine, provider, api_key)
    elif engine == "gpt":
        provider['base_url'] = BaseAPI(provider['base_url']).chat_url
        return await get_gpt_payload(request, engine, provider, api_key)
    elif engine == "openrouter":
        return await get_openrouter_payload(request, engine, provider, api_key)
    elif engine == "cloudflare":
        return await get_cloudflare_payload(request, engine, provider, api_key)
    elif engine == "cohere":
        return await get_cohere_payload(request, engine, provider, api_key)
    elif engine == "dalle":
        return await get_dalle_payload(request, engine, provider, api_key)
    elif engine == "whisper":
        return await get_whisper_payload(request, engine, provider, api_key)
    elif engine == "tts":
        return await get_tts_payload(request, engine, provider, api_key)
    elif engine == "moderation":
        return await get_moderation_payload(request, engine, provider, api_key)
    elif engine == "embedding":
        return await get_embedding_payload(request, engine, provider, api_key)
    else:
        raise ValueError("Unknown payload")

async def prepare_request_payload(provider, request_data):

    model_dict = get_model_dict(provider)
    request = RequestModel(**request_data)

    original_model = model_dict[request.model]
    engine, _ = get_engine(provider, endpoint=None, original_model=original_model)

    url, headers, payload = await get_payload(request, engine, provider, api_key=provider['api'])

    return url, headers, payload, engine
